@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id:  qt/helpwindow.cpp   3.0.5   edited Oct 12 2001 $
+** $Id: qt/helpwindow.cpp   3.3.5   edited Aug 31 12:13 $
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -66,9 +66,9 @@ HelpWindow::HelpWindow( const QString& home_, const QString& _path,
     file->insertItem( tr("&Print"), this, SLOT( print() ), ALT | Key_P );
 
     // The same three icons are used twice each.
-    QIconSet icon_back( QPixmap("textdrawing/back.xpm") );
-    QIconSet icon_forward( QPixmap("textdrawing/forward.xpm") );
-    QIconSet icon_home( QPixmap("textdrawing/home.xpm") );
+    QIconSet icon_back( QPixmap("textdrawing/previous.png") );
+    QIconSet icon_forward( QPixmap("textdrawing/next.png") );
+    QIconSet icon_home( QPixmap("textdrawing/home.png") );
 
     QPopupMenu* go = new QPopupMenu( this );
     backwardId = go->insertItem( icon_back,
