@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id: qt/qfileiconview.cpp   3.2.3   edited Oct 28 15:40 $
+** $Id: qt/qfileiconview.cpp   3.3.5   edited Aug 31 12:13 $
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -290,7 +290,14 @@ void QtFileIconDrag::append( const QIconDragItem &item, const QRect &pr,
 			     const QRect &tr, const QString &url )
 {
     QIconDrag::append( item, pr, tr );
-    urls << url;
+    QString ourUrl = url;    
+#ifdef Q_WS_WIN
+    if (ourUrl.length() > 2 && ourUrl[1] != ':') {
+	QDir dir(ourUrl);
+	ourUrl = dir.absPath();
+    }
+#endif
+    urls << QUriDrag::localFileToUri(ourUrl);
 }
 
 /*****************************************************************************
