@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id:  qt/showimg.h   3.0.5   edited Oct 12 2001 $
+** $Id: qt/showimg.h   3.3.5   edited Aug 31 12:17 $
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -25,7 +25,7 @@ class ImageViewer : public QWidget
 public:
     ImageViewer( QWidget *parent=0, const char *name=0, int wFlags=0 );
     ~ImageViewer();
-    bool	loadImage( const char *fileName );
+    bool	loadImage( const QString& );
 protected:
     void	paintEvent( QPaintEvent * );
     void	resizeEvent( QResizeEvent * );
@@ -40,7 +40,7 @@ private:
     bool	useColorContext() const;
     int		alloc_context;
     bool	convertEvent( QMouseEvent* e, int& x, int& y );
-    const char* filename;
+    QString     filename;
     QImage	image;			// the loaded image
     QPixmap	pm;			// the converted pixmap
     QPixmap	pmScaled;		// the scaled pixmap
