@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id: qt/textedit.h   3.2.3   edited May 13 09:08 $
+** $Id: qt/textedit.h   3.3.5   edited Aug 31 12:17 $
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -54,7 +54,6 @@ private slots:
     void textItalic();
     void textFamily( const QString &f );
     void textSize( const QString &p );
-    void textStyle( int s );
     void textColor();
     void textAlign( QAction *a );
 
@@ -72,7 +71,7 @@ private:
 	*actionAlignCenter,
 	*actionAlignRight,
 	*actionAlignJustify;
-    QComboBox *comboStyle,
+    QComboBox
 	*comboFont,
 	*comboSize;
     QTabWidget *tabWidget;
