@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id:  qt/xform.cpp   3.0.5   edited Oct 12 2001 $
+** $Id: qt/xform.cpp   3.3.5   edited Aug 31 12:13 $
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -377,7 +377,7 @@ void ShowXForm::showIt()
 	br.moveTopLeft( QPoint( -br.width()/2, -br.height()/2 ) );
         break;
       case Image:
-	r = pix.rect();
+	r = QRect(0, 0, pix.width()+1, pix.height()+1);
         break;
       case Picture:
 	// ### need QPicture::boundingRect()
@@ -385,12 +385,12 @@ void ShowXForm::showIt()
         break;
     }
     r.moveTopLeft( QPoint(-r.width()/2, -r.height()/2) );
+    r.moveBy( -1, -1 ); // add border for matrix round off
+    r.setSize( QSize( r.width() + 2,r.height() + 2 ) );
 	  // compute union of new and old rect
 	  // the resulting rectangle will cover what is already displayed
 	  // and have room for the new text/pixmap
-    eraseRect = eraseRect.unite( mtx.map(r) );
-    eraseRect.moveBy( -1, -1 ); // add border for matrix round off
-    eraseRect.setSize( QSize( eraseRect.width() + 2,eraseRect.height() + 2 ) );
+    eraseRect = eraseRect.unite( mtx.mapRect(r) );
     int pw = QMIN(eraseRect.width(),width());
     int ph = QMIN(eraseRect.height(),height());
     QPixmap pm( pw, ph );		// off-screen drawing pixmap
@@ -409,11 +409,8 @@ void ShowXForm::showIt()
 	p.drawRect( br );
 #endif
 	break;
-      case Image:
+    case Image:
 	p.drawPixmap( -pix.width()/2, -pix.height()/2, pix );
-	//QPixmap rotated = pix.xForm(mtx);
-	//bitBlt( &pm, pm.width()/2 - rotated.width()/2,
-		//pm.height()/2 - rotated.height()/2, &rotated );
 	break;
       case Picture:
 	// ### need QPicture::boundingRect()
