@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qaction.h   3.2.3   edited May 13 09:08 $
+** $Id: qt/qaction.h   3.3.5   edited Aug 31 12:17 $
 **
 ** Definition of QAction class
 **
 ** Created : 000000
 **
-** Copyright (C) 2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -116,9 +116,11 @@ protected:
     virtual void addedTo( int index, QPopupMenu *menu );
 
 public slots:
+    void activate();
     void toggle();
     virtual void setOn( bool );
     virtual void setEnabled( bool );
+    void setDisabled( bool );
     void setVisible( bool );
 
 signals:
@@ -189,6 +191,7 @@ private slots:
     void childToggled( bool );
     void childDestroyed();
     void internalComboBoxActivated( int );
+    void internalComboBoxHighlighted( int );
     void internalToggle( QAction* );
     void objectDestroyed();
 
