@@ -1,10 +1,10 @@
 /****************************************************************************
-** $Id: qt/qmlined.h   3.2.1   edited May 13 09:08 $
+** $Id: qt/qmlined.h   3.3.5   edited Aug 31 12:17 $
 **
 ** Compatibility file - should only be included by legacy code.
 ** It #includes the file which obsoletes this one.
 **
-** Copyright (C) 1998-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1998-2005 Trolltech AS.  All rights reserved.
 **  This file is part of the Qt GUI Toolkit.
 **
 ** This file may be distributed under the terms of the Q Public License
@@ -22,5 +22,5 @@
 *****************************************************************************/
 #ifndef QMLINED_H
 #define QMLINED_H
-#include "qtmultilineedit.h"
+#include "qmultilineedit.h"
 #endif
