@@ -1,9 +1,9 @@
 /****************************************************************************
-** $Id: qt/unixmake2.cpp   3.2.3   edited Sep 15 07:14 $
+** $Id: qt/unixmake2.cpp   3.3.5   edited Aug 31 12:13 $
 **
 ** Implementation of UnixMakefileGenerator class.
 **
-** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of qmake.
 **
@@ -41,12 +41,6 @@
 #include <qdir.h>
 #include <time.h>
 
-#ifdef Q_OS_WIN32
-#define QT_POPEN _popen
-#else
-#define QT_POPEN popen
-#endif
-
 QString mkdir_p_asstring(const QString &dir);
 
 UnixMakefileGenerator::UnixMakefileGenerator(QMakeProject *p) : MakefileGenerator(p), init_flag(FALSE), include_deps(FALSE)
@@ -78,6 +72,11 @@ UnixMakefileGenerator::writeMakefile(QTextStream &t)
     writeHeader(t);
     if(!project->variables()["QMAKE_FAILED_REQUIREMENTS"].isEmpty()) {
 	t << "QMAKE    = "	<< (project->isEmpty("QMAKE_QMAKE") ? QString("qmake") : var("QMAKE_QMAKE")) << endl;
+	{ //write the extra unix targets..
+	    QStringList &qut = project->variables()["QMAKE_EXTRA_UNIX_TARGETS"];
+	    for(QStringList::ConstIterator it = qut.begin(); it != qut.end(); ++it)
+		t << *it << " ";
+	}
 	t << "all clean install distclean mocables uninstall uicables:" << "\n\t"
 	  << "@echo \"Some of the required modules ("
 	  << var("QMAKE_FAILED_REQUIREMENTS") << ") are not available.\"" << "\n\t"
@@ -187,6 +186,9 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
     t << "COPY     = " << var("QMAKE_COPY") << endl;
     t << "COPY_FILE= " << var("QMAKE_COPY_FILE") << endl;
     t << "COPY_DIR = " << var("QMAKE_COPY_DIR") << endl;
+    t << "INSTALL_FILE= " << var("QMAKE_INSTALL_FILE") << endl;
+    t << "INSTALL_DIR = " << var("QMAKE_INSTALL_DIR") << endl;
+
     t << "DEL_FILE = " << var("QMAKE_DEL_FILE") << endl;
     t << "SYMLINK  = " << var("QMAKE_SYMBOLIC_LINK") << endl;
     t << "DEL_DIR  = " << var("QMAKE_DEL_DIR") << endl;
@@ -365,7 +367,7 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 			QStringList deps = findDependencies((*it)).grep(QRegExp(Option::cpp_moc_ext + "$"));
 			if(!deps.isEmpty())
 			    t << d_file << ": " << deps.join(" ") << endl;
-			t << "-include " << d_file << endl;
+			t << var("QMAKE_CFLAGS_USE_PRECOMPILE") << " " << d_file << endl;
 		    }
 		}
 	    }
@@ -462,7 +464,7 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 		t << "\n\t" << "test -d " << destdir << " || mkdir -p " << destdir << "\n\t";
 	    if(!project->isEmpty("QMAKE_PRE_LINK"))
 		t << var("QMAKE_PRE_LINK") << "\n\t";
-	    t << "$(LINK) $(LFLAGS) -o $(TARGET) " << incr_deps << " " << incr_objs << " $(LIBS) $(OBJCOMP)";
+	    t << "$(LINK) $(LFLAGS) -o $(TARGET) " << incr_deps << " " << incr_objs << " $(OBJCOMP) $(LIBS)";
 	    if(!project->isEmpty("QMAKE_POST_LINK"))
 		t << "\n\t" << var("QMAKE_POST_LINK");
 	    t << endl << endl;
@@ -476,7 +478,7 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 		t << "test -d " << destdir << " || mkdir -p " << destdir << "\n\t";
 	    if(!project->isEmpty("QMAKE_PRE_LINK"))
 		t << var("QMAKE_PRE_LINK") << "\n\t";
-	    t << "$(LINK) $(LFLAGS) -o $(TARGET) $(OBJECTS) $(OBJMOC) $(LIBS) $(OBJCOMP)";
+	    t << "$(LINK) $(LFLAGS) -o $(TARGET) $(OBJECTS) $(OBJMOC) $(OBJCOMP) $(LIBS)";
 	    if(!project->isEmpty("QMAKE_POST_LINK"))
 		t << "\n\t" << var("QMAKE_POST_LINK");
 	    t << endl << endl;
@@ -681,7 +683,7 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 	fixEnvVariables(moc);
 	if(target != moc)
 	    t << "$(MOC): \n\t"
-	      << "( cd " << moc_dir << " ; $(MAKE) )"  << endl << endl;
+	      << "( cd " << moc_dir << " && $(MAKE) )"  << endl << endl;
     }
 
     writeMakeQmake(t);
@@ -726,10 +728,11 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 	  << ",g\" \"" << info_plist << "\" >\"" << info_plist_out << "\"" << endl;
 	if(!project->first("RC_FILE").isEmpty()) {
 	    QString dir = destdir + "../Resources/";
-	    t << dir << "application.icns: " << var("RC_FILE") << "\n\t"
+	    t << dir << "application.icns: " << fileFixify(var("RC_FILE")) << "\n\t"
 	      << "@test -d " << dir << " || mkdir -p " << dir << "\n\t"
 	      << "@$(DEL_FILE) " << dir << "application.icns" << "\n\t"
-	      << "@$(COPY_FILE) " << var("RC_FILE") << " " << dir << "application.icns" << endl;
+	      << "@$(COPY_FILE) " << fileFixify(var("RC_FILE"))
+              << " " << dir << "application.icns" << endl;
 	}
     }
 
@@ -837,6 +840,12 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 	else
 	    t << "-$(DEL_FILE) $(OBJECTS)" << "\n\t";
     }
+    if(doPrecompiledHeaders() && !project->isEmpty("PRECOMPILED_HEADER")) {
+	QString header_prefix = project->first("QMAKE_PRECOMP_PREFIX");
+	QString precomph_out_dir = project->first("QMAKE_ORIG_TARGET") + ".gch" + Option::dir_sep;
+	t << "-$(DEL_FILE) " << precomph_out_dir << (header_prefix + "c ")
+	  << precomph_out_dir << header_prefix << "c++" << "\n\t";
+    }
     if(!project->isEmpty("IMAGES"))
 	t << varGlue("QMAKE_IMAGE_COLLECTION", "\t-$(DEL_FILE) ", " ", "") << "\n\t";
     if(src_incremental)
@@ -872,19 +881,41 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 	  << destdir << "$(TARGET2) $(TARGETA)" << "\n";
     t << endl << endl;
 
-    if ( !project->isEmpty("PRECOMPH") ) {
+    if(doPrecompiledHeaders() && !project->isEmpty("PRECOMPILED_HEADER") ) {
+	QString precomph = fileFixify(project->first("PRECOMPILED_HEADER"));
+	t << "###### Prefix headers" << endl;
+	QString comps[] = { "C", "CXX", QString::null };
+	for(int i = 0; !comps[i].isNull(); i++) {
+	    QString flags = var("QMAKE_" + comps[i] + "FLAGS_PRECOMPILE");
+	    flags += " $(" + comps[i] + "FLAGS)";
+
+	    QString header_prefix = project->first("QMAKE_PRECOMP_PREFIX");
+	    QString outdir = project->first("QMAKE_ORIG_TARGET") + ".gch" + Option::dir_sep, outfile = outdir;
+	    QString compiler;
+	    if(comps[i] == "C") {
+		outfile += header_prefix + "c";
+		compiler = "$(CC) ";
+	    } else {
+		outfile += header_prefix + "c++";
+		compiler = "$(CXX) ";
+	    }
+	    t << outfile << ": " << precomph << " " << findDependencies(precomph).join(" \\\n\t\t")
+	      << "\n\t" << "test -d " << outdir << " || mkdir -p " << outdir
+	      << "\n\t" << compiler << flags << " $(INCPATH) " << precomph << " -o " << outfile << endl << endl;
+	}
+    }
+    if(!project->isEmpty("ALLMOC_HEADER")) {
 	QString outdir = project->first("MOC_DIR");
-	QString qt_dot_h = Option::fixPathToLocalOS(project->first("PRECOMPH"));
-	t << "###### Combined headers" << endl << endl;
-	//XXX
-	t << outdir << "allmoc.cpp: " << qt_dot_h << " "
+	QString precomph = fileFixify(project->first("ALLMOC_HEADER"));
+	t << "###### Combined headers" << endl << endl
+	  << outdir << "allmoc.cpp: " << precomph << " "
 	  << varList("HEADERS_ORIG") << "\n\t"
-	  << "echo '#include \"" << qt_dot_h << "\"' >" << outdir << "allmoc.cpp" << "\n\t"
+	  << "echo '#include \"" << precomph << "\"' >" << outdir << "allmoc.cpp" << "\n\t"
 	  << "$(CXX) -E -DQT_MOC_CPP -DQT_NO_STL $(CXXFLAGS) $(INCPATH) >" << outdir << "allmoc.h "
-	     << outdir << "allmoc.cpp" << "\n\t"
+	  << outdir << "allmoc.cpp" << "\n\t"
 	  << "$(MOC) -o " << outdir << "allmoc.cpp " << outdir << "allmoc.h" << "\n\t"
 	  << "perl -pi -e 's{#include \"allmoc.h\"}{#define QT_H_CPP\\n#include \""
-	     << qt_dot_h << "\"}' " << outdir << "allmoc.cpp" << "\n\t"
+	  << precomph << "\"}' " << outdir << "allmoc.cpp" << "\n\t"
 	  << "$(DEL_FILE) " << outdir << "allmoc.h" << endl << endl;
     }
 
@@ -924,19 +955,19 @@ UnixMakefileGenerator::writeMakeParts(QTextStream &t)
 		QString in = Option::fixPathToTargetOS((*input), FALSE),
 		       out = tmp_out, cmd = tmp_cmd, deps;
 		out.replace("${QMAKE_FILE_BASE}", fi.baseName());
-		out.replace("${QMAKE_FILE_NAME}", fi.fileName());
+		out.replace("${QMAKE_FILE_NAME}", fi.filePath());
 		cmd.replace("${QMAKE_FILE_BASE}", fi.baseName());
 		cmd.replace("${QMAKE_FILE_OUT}", out);
-		cmd.replace("${QMAKE_FILE_NAME}", fi.fileName());
+		cmd.replace("${QMAKE_FILE_NAME}", fi.filePath());
 		for(QStringList::Iterator it3 = vars.begin(); it3 != vars.end(); ++it3)
 		    cmd.replace("$(" + (*it3) + ")", "$(QMAKE_COMP_" + (*it3)+")");
 		if(!tmp_dep.isEmpty()) {
 		    char buff[256];
 		    QString dep_cmd = tmp_dep;
-		    dep_cmd.replace("${QMAKE_FILE_NAME}", fi.fileName());
+		    dep_cmd.replace("${QMAKE_FILE_NAME}", fi.filePath());
 		    if(FILE *proc = QT_POPEN(dep_cmd.latin1(), "r")) {
 			while(!feof(proc)) {
-			    int read_in = fread(buff, 1, 255, proc);
+			    int read_in = int(fread(buff, 1, 255, proc));
 			    if(!read_in)
 				break;
 			    int l = 0;
@@ -1019,6 +1050,8 @@ UnixMakefileGenerator::writeSubdirs(QTextStream &t, bool direct)
     t << "DEL_FILE =    " << var("QMAKE_DEL_FILE") << endl;
     t << "CHK_DIR_EXISTS= " << var("QMAKE_CHK_DIR_EXISTS") << endl;
     t << "MKDIR    = " << var("QMAKE_MKDIR") << endl;
+    t << "INSTALL_FILE= " << var("QMAKE_INSTALL_FILE") << endl;
+    t << "INSTALL_DIR = " << var("QMAKE_INSTALL_DIR") << endl;
     t << "SUBTARGETS =	";     // subdirectory targets are sub-directory
     for( it.toFirst(); it.current(); ++it)
 	t << " \\\n\t\t" << it.current()->target;
@@ -1079,7 +1112,7 @@ UnixMakefileGenerator::writeSubdirs(QTextStream &t, bool direct)
 	      << " && $(MAKE) -f " << (*it)->makefile << " qmake_all" << "; ) || true";
 	}
 	t << endl;
-	t << "clean uicables mocables uiclean mocclean lexclean yaccclean " 
+	t << "clean uicables mocables uiclean mocclean lexclean yaccclean "
 	  << var("SUBDIR_TARGETS") << ": qmake_all FORCE";
 	for( it.toFirst(); it.current(); ++it) {
 	    t << "\n\t ( ";
@@ -1135,8 +1168,10 @@ UnixMakefileGenerator::writeSubdirs(QTextStream &t, bool direct)
 	}
 	if(project->variables()[(*qut_it) + ".CONFIG"].findIndex("phony") != -1)
 	    deps += QString(" ") + "FORCE";
-	t << targ << ":" << deps << "\n\t"
-	  << cmd << endl << endl;
+	t << targ << ":" << deps << "\n";
+        if(!cmd.isEmpty())
+            t << "\t" << cmd << endl;
+        t << endl;
     }
     t <<"FORCE:" << endl << endl;
 }
@@ -1185,15 +1220,15 @@ void UnixMakefileGenerator::init2()
 	    project->variables()["TARGET"] = project->variables()["TARGET_la"];
 	} else if( project->isActiveConfig("plugin") ) {
 	    project->variables()["TARGET_x.y.z"].append("lib" +
-							project->first("TARGET") + "." + 
-							project->first("QMAKE_EXTENSION_SHLIB"));
+							project->first("TARGET") + "." +
+							project->first("QMAKE_EXTENSION_PLUGIN"));
 	    if(project->isActiveConfig("lib_version_first"))
 		project->variables()["TARGET_x"].append("lib" + project->first("TARGET") + "." +
 							project->first("VER_MAJ") + "." +
-							project->first("QMAKE_EXTENSION_SHLIB"));
+							project->first("QMAKE_EXTENSION_PLUGIN"));
 	    else
 		project->variables()["TARGET_x"].append("lib" + project->first("TARGET") + "." +
-							project->first("QMAKE_EXTENSION_SHLIB") +
+							project->first("QMAKE_EXTENSION_PLUGIN") +
 							"." + project->first("VER_MAJ"));
 
 	    project->variables()["TARGET"] = project->variables()["TARGET_x.y.z"];
@@ -1293,11 +1328,17 @@ void UnixMakefileGenerator::init2()
     if(project->isEmpty("QMAKE_SYMBOLIC_LINK"))
 	project->variables()["QMAKE_SYMBOLIC_LINK"].append("ln -sf");
     if ( !project->variables()["QMAKE_APP_FLAG"].isEmpty() ) {
+	project->variables()["QMAKE_CFLAGS"] += project->variables()["QMAKE_CFLAGS_APP"];
+	project->variables()["QMAKE_CXXFLAGS"] += project->variables()["QMAKE_CXXFLAGS_APP"];
 	project->variables()["QMAKE_LFLAGS"] += project->variables()["QMAKE_LFLAGS_APP"];
     } else if ( project->isActiveConfig("dll") ) {
-	project->variables()["QMAKE_CFLAGS"] += project->variables()["QMAKE_CFLAGS_SHLIB"];
-	project->variables()["QMAKE_CXXFLAGS"] += project->variables()["QMAKE_CXXFLAGS_SHLIB"];
+	if( !project->isActiveConfig("plugin") || !project->isActiveConfig("plugin_no_share_shlib_cflags")) {
+	    project->variables()["QMAKE_CFLAGS"] += project->variables()["QMAKE_CFLAGS_SHLIB"];
+	    project->variables()["QMAKE_CXXFLAGS"] += project->variables()["QMAKE_CXXFLAGS_SHLIB"];
+	}
 	if ( project->isActiveConfig("plugin") ) {
+	    project->variables()["QMAKE_CFLAGS"] += project->variables()["QMAKE_CFLAGS_PLUGIN"];
+	    project->variables()["QMAKE_CXXFLAGS"] += project->variables()["QMAKE_CXXFLAGS_PLUGIN"];
 	    project->variables()["QMAKE_LFLAGS"] += project->variables()["QMAKE_LFLAGS_PLUGIN"];
 	    if( project->isActiveConfig("plugin_with_soname") && !project->isActiveConfig("compile_libtool"))
 		project->variables()["QMAKE_LFLAGS"] += project->variables()["QMAKE_LFLAGS_SONAME"];
@@ -1349,7 +1390,7 @@ void UnixMakefileGenerator::init2()
 		QString in = Option::fixPathToTargetOS((*input), FALSE),
 		       out = tmp_out;
 		out.replace("${QMAKE_FILE_BASE}", fi.baseName());
-		out.replace("${QMAKE_FILE_NAME}", fi.fileName());
+		out.replace("${QMAKE_FILE_NAME}", fi.filePath());
 		if(project->variables()[(*it) + ".CONFIG"].findIndex("no_link") == -1)
 		    project->variables()["OBJCOMP"] += out;
 	    }
@@ -1379,14 +1420,14 @@ UnixMakefileGenerator::writeLibtoolFile()
     QString fname = libtoolFileName(), lname = fname;
     int slsh = lname.findRev(Option::dir_sep);
     if(slsh != -1)
-	lname = lname.right(lname.length() - slsh);
+	lname = lname.right(lname.length() - slsh - 1);
     QFile ft(fname);
     if(!ft.open(IO_WriteOnly))
 	return;
     project->variables()["ALL_DEPS"].append(fname);
 
     QTextStream t(&ft);
-    t << "# " << lname << Option::libtool_ext << " - a libtool library file\n";
+    t << "# " << lname << " - a libtool library file\n";
     time_t now = time(NULL);
     t << "# Generated by qmake/libtool (" << qmake_version() << ") (Qt "
       << QT_VERSION_STR << ") on: " << ctime(&now) << "\n";
@@ -1407,7 +1448,7 @@ UnixMakefileGenerator::writeLibtoolFile()
     t << "'\n\n";
 
     t << "# The name of the static archive.\n"
-      << "old_library='" << lname << ".a'\n\n";
+      << "old_library='" << lname.left(lname.length()-Option::libtool_ext.length()) << ".a'\n\n";
 
     t << "# Libraries that this one depends upon.\n";
     QStringList libs;
@@ -1462,33 +1503,67 @@ UnixMakefileGenerator::pkgConfigFileName()
     return ret;
 }
 
+QString
+UnixMakefileGenerator::pkgConfigPrefix() const
+{
+    if(!project->isEmpty("QMAKE_PKGCONFIG_PREFIX"))
+	return project->first("QMAKE_PKGCONFIG_PREFIX");
+    return qInstallPath();
+}
+
+QString
+UnixMakefileGenerator::pkgConfigFixPath(QString path) const
+{
+    QString prefix = pkgConfigPrefix();
+    if(path.startsWith(prefix))
+	path = path.replace(prefix, "${prefix}");
+    return path;
+}
+
 void
 UnixMakefileGenerator::writePkgConfigFile()     // ### does make sense only for libqt so far
 {
     QString fname = pkgConfigFileName(), lname = fname;
     int slsh = lname.findRev(Option::dir_sep);
     if(slsh != -1)
-	lname = lname.right(lname.length() - slsh);
+	lname = lname.right(lname.length() - slsh - 1);
     QFile ft(fname);
     if(!ft.open(IO_WriteOnly))
 	return;
     project->variables()["ALL_DEPS"].append(fname);
     QTextStream t(&ft);
 
-    QString prefix = qInstallPath();
-    QString libDir = prefix + "/lib";
-    QString includeDir = prefix + "/include";
+    QString prefix = pkgConfigPrefix();
+    QString libDir = project->first("QMAKE_PKGCONFIG_LIBDIR");
+    if(libDir.isEmpty())
+	libDir = prefix + "/lib";
+    QString includeDir = project->first("QMAKE_PKGCONFIG_INCDIR");
+    if(includeDir.isEmpty())
+	includeDir = prefix + "/include";
 
-    t << "prefix=" << libDir << endl;
+    t << "prefix=" << prefix << endl;
     t << "exec_prefix=${prefix}\n"
-      << "libdir=${exec_prefix}/lib\n"
-      << "includedir=${prefix}/include" << endl;
+      << "libdir=" << pkgConfigFixPath(libDir) << "\n"
+      << "includedir=" << pkgConfigFixPath(includeDir) << endl;
     // non-standard entry. Provides useful info normally only
     // contained in the internal .qmake.cache file
     t << varGlue("CONFIG", "qt_config=", " ", "") << endl << endl;
 
     t << "Name: Qt" << endl;
-    t << "Description: Qt GUI Library" << endl;
+    QString desc = project->first("QMAKE_PKGCONFIG_DESCRIPTION");
+    if(desc.isEmpty()) {
+	desc = project->first("TARGET").lower();
+	desc.replace(0, 1, desc[0].upper());
+	if(project->first("TEMPLATE") == "lib") {
+	    if(project->isActiveConfig("plugin"))
+	       desc += " Plugin";
+	    else
+	       desc += " Library";
+	} else if(project->first("TEMPLATE") == "app") {
+	    desc += " Application";
+	}
+    }
+    t << "Description: " << desc << endl;
     t << "Version: " << project->first("VERSION") << endl;
 
     // libs
@@ -1499,7 +1574,7 @@ UnixMakefileGenerator::writePkgConfigFile()     // ### does make sense only for
 	libs << "QMAKE_LIBS"; //obvious one
     if(project->isActiveConfig("thread"))
 	libs << "QMAKE_LFLAGS_THREAD"; //not sure about this one, but what about things like -pthread?
-    t << "Libs: -L" << libDir << " -l" << lname << " ";
+    t << "Libs: -L${libdir} -l" << lname.left(lname.length()-Option::libtool_ext.length()) << " ";
     for(QStringList::ConstIterator it = libs.begin(); it != libs.end(); ++it)
 	t << project->variables()[(*it)].join(" ") << " ";
     t << endl;
@@ -1511,5 +1586,5 @@ UnixMakefileGenerator::writePkgConfigFile()     // ### does make sense only for
       << varGlue("PRL_EXPORT_DEFINES","-D"," -D"," ")
       << project->variables()["PRL_EXPORT_CXXFLAGS"].join(" ")
 	//      << varGlue("DEFINES","-D"," -D"," ")
-      << " -I" << includeDir;
+      << " -I${includedir}";
 }
