@@ -1,9 +1,9 @@
 /****************************************************************************
-** $Id: qt/main.cpp   3.3.3   edited May 30 2003 $
+** $Id: qt/main.cpp   3.3.5   edited Aug 31 12:13 $
 **
 ** ???
 **
-** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of qmake.
 **
@@ -54,8 +54,17 @@
 bool qt_resolve_symlinks = FALSE;
 #endif
 
+#if defined(Q_WS_WIN)
+extern Q_EXPORT int qt_ntfs_permission_lookup;
+#endif
+
 int main(int argc, char **argv)
 {
+#if defined(Q_WS_WIN)
+    // Workaround for QFileInfo::isReadable() failing for certain users. See task: 54320
+    qt_ntfs_permission_lookup = 0;
+#endif
+
     /* parse command line */
     if(!Option::parseCommandLine(argc, argv))
 	return 666;
