@@ -1,9 +1,9 @@
 /****************************************************************************
-** $Id: qt/project.cpp   3.2.3   edited Oct 24 03:34 $
+** $Id: qt/project.cpp   3.3.5   edited Aug 31 12:13 $
 **
 ** Implementation of QMakeProject class.
 **
-** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of qmake.
 **
@@ -122,6 +122,12 @@ static QString varMap(const QString &x)
 	ret = "QMAKE_H_MOD_MOC";
     else if(ret == "QMAKE_LFLAGS_SHAPP")
 	ret = "QMAKE_LFLAGS_APP";
+    else if(ret == "PRECOMPH")
+	ret = "PRECOMPILED_HEADER";
+    else if(ret == "PRECOMPCPP")
+	ret = "PRECOMPILED_SOURCE";
+    else if(ret == "INCPATH")
+	ret = "INCLUDEPATH";
     return ret;
 }
 
@@ -132,8 +138,11 @@ static QStringList split_arg_list(const QString &params)
     for(int x = 0, last = 0, parens = 0; x <= (int)params.length(); x++) {
 	if(x == (int)params.length()) {
 	    QString mid = params.mid(last, x - last).stripWhiteSpace();
-	    if(mid[(int)mid.length()-1] == quote)
-		mid.truncate(1);
+            if(quote) {
+                if(mid[(int)mid.length()-1] == quote)
+                    mid.truncate(1);
+                quote = 0;
+            }
 	    args << mid;
 	} else if(params[x] == ')') {
 	    parens--;
@@ -141,7 +150,7 @@ static QStringList split_arg_list(const QString &params)
 	    parens++;
 	} else if(params[x] == quote) {
 	    quote = 0;
-	} else if(params[x] == '\'' || params[x] == '"') {
+	} else if(!quote && (params[x] == '\'' || params[x] == '"')) {
 	    quote = params[x];
 	} else if(!parens && !quote && params[x] == ',') {
 	    args << params.mid(last, x - last).stripWhiteSpace();
@@ -186,6 +195,14 @@ QMakeProject::QMakeProject(QMakeProperty *p)
     prop = p;
 }
 
+void
+QMakeProject::reset()
+{
+    /* scope blocks start at true */
+    test_status = TestNone;
+    scope_flag = 0x01;
+    scope_block = 0;
+}
 
 bool
 QMakeProject::parse(const QString &t, QMap<QString, QStringList> &place)
@@ -270,7 +287,7 @@ QMakeProject::parse(const QString &t, QMap<QString, QStringList> &place)
 		if(invert_test)
 		    comp_scope = comp_scope.right(comp_scope.length()-1);
 		int lparen = comp_scope.find('(');
-		if(or_op || !scope_failed) {
+		if(or_op == scope_failed) {
 		    if(lparen != -1) { /* if there is an lparen in the scope, it IS a function */
 			int rparen = comp_scope.findRev(')');
 			if(rparen == -1) {
@@ -327,8 +344,6 @@ QMakeProject::parse(const QString &t, QMap<QString, QStringList> &place)
 	test_status = TestNone;
     else if(!else_line || test_status != TestFound)
 	test_status = (scope_failed ? TestSeek : TestFound);
-    if(scope_failed)
-	return TRUE; /* oh well */
     if(!*d) {
 	if(!var.stripWhiteSpace().isEmpty())
 	    qmake_error_msg("Parse Error ('" + s + "')");
@@ -354,7 +369,8 @@ QMakeProject::parse(const QString &t, QMap<QString, QStringList> &place)
 		 vals.latin1(), parser.file.latin1(), parser.line_no);
     }
     doVariableReplace(vals, place);
-
+    if(scope_failed)
+	return TRUE; /* oh well */
 #undef SKIP_WS
 
     if(!var.isEmpty() && Option::mkfile::do_preprocess) {
@@ -439,10 +455,7 @@ bool
 QMakeProject::read(const QString &file, QMap<QString, QStringList> &place)
 {
     parser_info pi = parser;
-    /* scope blocks start at true */
-    test_status = TestNone;
-    scope_flag = 0x01;
-    scope_block = 0;
+    reset();
 
     QString filename = Option::fixPathToLocalOS(file);
     doVariableReplace(filename, place);
@@ -490,6 +503,9 @@ QMakeProject::read(const QString &file, QMap<QString, QStringList> &place)
 	    qfile.close();
     }
     parser = pi;
+    if(scope_block != 0)
+	warn_msg(WarnParser, "%s: Unterminated conditional at end of file.",
+		 file.latin1());
     return ret;
 }
 
@@ -537,7 +553,8 @@ QMakeProject::read(uchar cmd)
 	    if(Option::mkfile::qmakespec.isEmpty()) {
 		for(QStringList::Iterator it = mkspec_roots.begin(); it != mkspec_roots.end(); ++it) {
 		    QString mkspec = (*it) + QDir::separator() + "default";
-		    if(QFile::exists(mkspec)) {
+                    QFileInfo default_info(mkspec);
+                    if(default_info.exists() && default_info.isSymLink()) {
 			Option::mkfile::qmakespec = mkspec;
 			break;
 		    }
@@ -587,6 +604,7 @@ QMakeProject::read(uchar cmd)
 	    cfile = pfile;
 	    parser.line_no = 1; //really arg count now.. duh
 	    parser.file = "(internal)";
+	    reset();
 	    for(QStringList::Iterator it = Option::before_user_vars.begin();
 		it != Option::before_user_vars.end(); ++it) {
 		if(!parse((*it), base_vars)) {
@@ -617,6 +635,7 @@ QMakeProject::read(uchar cmd)
     if(cmd & ReadCmdLine) {
 	parser.line_no = 1; //really arg count now.. duh
 	parser.file = "(internal)";
+	reset();
 	for(QStringList::Iterator it = Option::after_user_vars.begin();
 	    it != Option::after_user_vars.end(); ++it) {
 	    if(!parse((*it), vars)) {
@@ -935,7 +954,7 @@ QMakeProject::doProjectTest(const QString& func, QStringList args, QMap<QString,
 	    QStringList include_roots;
 	    include_roots << Option::output_dir;
 	    QString pfilewd = QFileInfo(parser.file).dirPath();
-	    if(pfilewd.isEmpty())
+	    if(!pfilewd.isEmpty())
 		include_roots << pfilewd;
 	    if(Option::output_dir != QDir::currentDirPath())
 		include_roots << QDir::currentDirPath();
@@ -1148,7 +1167,7 @@ QMakeProject::doVariableReplace(QString &str, const QMap<QString, QStringList> &
 		replacement = "$";
 	    else if(val == "LITERAL_HASH")
 		replacement = "#";
-	    else if(val == "PWD") 
+	    else if(val == "PWD")
 		replacement = QDir::currentDirPath();
 	    else if(val == "DIR_SEPARATOR")
 		replacement = Option::dir_sep;
@@ -1167,12 +1186,12 @@ QMakeProject::doVariableReplace(QString &str, const QMap<QString, QStringList> &
 		    if(arg_list.count() >= 2)
 			start = arg_list[1].toInt();
 		    if(start < 0)
-			start += var.count();
+			start += int(var.count());
 		    int end = start;
 		    if(arg_list.count() == 3)
 			end = arg_list[2].toInt();
 		    if(end < 0)
-			end += var.count();
+			end += int(var.count());
 		    if(end < start)
 			end = start;
 		    for(int i = start; i <= end && (int)var.count() >= i; i++) {
@@ -1209,7 +1228,7 @@ QMakeProject::doVariableReplace(QString &str, const QMap<QString, QStringList> &
 		    int di = file.findRev(Option::dir_sep);
 		    QDir sunworkshop42workaround = QDir::current();
 		    QString oldpwd = sunworkshop42workaround.currentDirPath();
-		    if(di != -1 && QDir::setCurrent(file.left(file.findRev(Option::dir_sep)))) 
+		    if(di != -1 && QDir::setCurrent(file.left(file.findRev(Option::dir_sep))))
 			file = file.right(file.length() - di - 1);
 		    parser_info pi = parser;
 		    int sb = scope_block;
@@ -1303,7 +1322,7 @@ QMakeProject::doVariableReplace(QString &str, const QMap<QString, QStringList> &
 		    char buff[256];
 		    FILE *proc = QT_POPEN(arg_list.join(" ").latin1(), "r");
 		    while(proc && !feof(proc)) {
-			int read_in = fread(buff, 1, 255, proc);
+			int read_in = int(fread(buff, 1, 255, proc));
 			if(!read_in)
 			    break;
 			for(int i = 0; i < read_in; i++) {
