@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qmessagebox.cpp   3.3.4   edited Aug 10 16:43 $
+** $Id: qt/qmessagebox.cpp   3.3.5   edited Aug 31 12:13 $
 **
 ** Implementation of QMessageBox class
 **
 ** Created : 950503
 **
-** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the dialogs module of the Qt GUI Toolkit.
 **
@@ -947,6 +947,9 @@ void QMessageBox::adjustSize()
         w = screen.width();
     resize( w, h );
     setMinimumSize( size() );
+#ifdef Q_WS_MAC
+    setMaximumSize(size());
+#endif
 }
 
 
@@ -974,22 +977,26 @@ void QMessageBox::resizeEvent( QResizeEvent * )
         btn_spacing = border;
     int lmargin = 0;
     mbd->iconLabel.adjustSize();
-    mbd->iconLabel.move( border, border );
+    bool rtl = QApplication::reverseLayout();
+    if (rtl)
+        mbd->iconLabel.move(width() - border - mbd->iconLabel.width(), border);
+    else
+        mbd->iconLabel.move(border, border);
     if ( mbd->iconLabel.pixmap() && mbd->iconLabel.pixmap()->width() )
         lmargin += mbd->iconLabel.width() + border;
-    label->setGeometry( lmargin+border,
-                        border,
-                        width() - lmargin -2*border,
-                        height() - 3*border - bh );
+    label->setGeometry((rtl ? 0 : lmargin) + border,
+                       border,
+                       width() - lmargin -2*border,
+                       height() - 3*border - bh);
     int extra_space = (width() - bw*n - 2*border - (n-1)*btn_spacing);
     if ( style().styleHint(QStyle::SH_GUIStyle) == MotifStyle )
         for ( i=0; i<n; i++ )
-            mbd->pb[i]->move( border + i*bw + i*btn_spacing + extra_space*(i+1)/(n+1),
-                              height() - border - bh );
+            mbd->pb[rtl ? n - i - 1 : i]->move(border + i*bw + i*btn_spacing + extra_space*(i+1)/(n+1),
+                                               height() - border - bh );
     else
         for ( i=0; i<n; i++ )
-            mbd->pb[i]->move( border + i*bw + extra_space/2 + i*btn_spacing,
-                              height() - border - bh );
+            mbd->pb[rtl ? n - i - 1 : i]->move(border + i*bw + extra_space/2 + i*btn_spacing,
+                                               height() - border - bh );
 }
 
 
