@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qaccel.cpp   3.2.3   edited Jul 4 13:16 $
+** $Id: qt/qaccel.cpp   3.3.5   edited Aug 31 12:13 $
 **
 ** Implementation of QAccel class
 **
 ** Created : 950419
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -165,8 +165,6 @@ public:
     void activateAmbiguously( QAccelItem* item );
 };
 
-
-
 class QAccelManager : public Qt {
 public:
     static QAccelManager* self() { return self_ptr ? self_ptr : new QAccelManager; }
@@ -204,13 +202,20 @@ bool Q_EXPORT qt_tryComposeUnicode( QWidget* w, QKeyEvent*  e){
     return QAccelManager::self()->tryComposeUnicode( w, e );
 }
 
+#ifdef Q_WS_MAC
+static bool qt_accel_no_shortcuts = TRUE;
+#else
+static bool qt_accel_no_shortcuts = FALSE;
+#endif
+void Q_EXPORT qt_setAccelAutoShortcuts(bool b) { qt_accel_no_shortcuts = b; }
+
 /*
     \internal
     Returns TRUE if the accel is in the current subwindow, else FALSE.
 */
 bool QAccelManager::correctSubWindow( QWidget* w, QAccelPrivate* d ) {
 #if !defined ( Q_OS_MACX )
-    if ( !d->watch || !d->watch->isVisible() || !d->watch->isEnabled() )
+     if ( !d->watch || !d->watch->isVisible() || !d->watch->isEnabled() )
 #else
     if ( !d->watch || (!d->watch->isVisible() && !d->watch->inherits( "QMenuBar" )) || !d->watch->isEnabled() )
 #endif
@@ -254,7 +259,6 @@ inline int QAccelManager::translateModifiers( ButtonState state )
     if ( state & AltButton )
 	result |= ALT;
     return result;
-
 }
 
 /*
@@ -462,13 +466,15 @@ bool QAccelManager::dispatchAccelEvent( QWidget* w, QKeyEvent* e )
 	if ( mainStatusBar && (PartialMatch == currentState || intermediate.count() ) ) {
 	    if ( currentState == Qt::PartialMatch ) {
 		mainStatusBar->message( (QString)partial + ", ...", 0 );
-	    } else {
+	    } else if (!identicalDisabled) {
 		QString message = QAccel::tr("%1, %2 not defined").
 		    arg( (QString)intermediate ).
 		    arg( QKeySequence::encodeString( e->key() | translateModifiers(e->state()) ) );
 		mainStatusBar->message( message, 2000 );
 		// Since we're a NoMatch, reset the clash count
 		clash = -1;
+	    } else {
+	    	mainStatusBar->clear();
 	    }
 	}
 #endif
@@ -856,6 +862,9 @@ void QAccelPrivate::activateAmbiguously( QAccelItem* item )
 
 QKeySequence QAccel::shortcutKey( const QString &str )
 {
+    if(qt_accel_no_shortcuts)
+	return QKeySequence();
+
     int p = 0;
     while ( p >= 0 ) {
 	p = str.find( '&', p ) + 1;
@@ -864,7 +873,11 @@ QKeySequence QAccel::shortcutKey( const QString &str )
 	if ( str[p] != '&' ) {
 	    QChar c = str[p];
 	    if ( c.isPrint() ) {
-		c = c.upper();
+	        char ltr = c.upper().latin1();
+	        if ( ltr >= (char)Key_A && ltr <= (char)Key_Z )
+                    c = ltr;
+                else
+                    c = c.lower();
 		return QKeySequence( c.unicode() + ALT + UNICODE_ACCEL );
 	    }
 	}
