@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qjpegio.h   3.0.5   edited Oct 12 2001 $
+** $Id: qt/qjpegio.h   3.3.5   edited Aug 31 12:17 $
 **
 ** Definition of JPEG QImage IOHandler
 **
 ** Created : 970621
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -38,7 +38,7 @@
 #ifndef QJPEGIO_H
 #define QJPEGIO_H
 
-#include <qglobal.h>
+#include "qglobal.h"
 
 #ifndef QT_NO_IMAGEIO_JPEG
 
