@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qpixmap_x11.cpp   3.2.3   edited May 13 14:31 $
+** $Id: qt/qpixmap_x11.cpp   3.3.5   edited Aug 31 12:13 $
 **
 ** Implementation of QPixmap class for X11
 **
 ** Created : 940501
 **
-** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -62,6 +62,10 @@
 
 #include <stdlib.h>
 
+#if defined(Q_CC_MIPS)
+#  define for if(0){}else for
+#endif
+
 
 /*!
   \class QPixmap::QPixmapData
@@ -305,6 +309,9 @@ void QPixmap::init( int w, int h, int d, bool bitmap, Optimization optim )
 
     int dd = x11Depth();
 
+    if ( d != -1 )
+        dd = d;
+
     if ( optim == DefaultOptim )		// use default optimization
 	optim = defOptim;
 
@@ -338,7 +345,7 @@ void QPixmap::init( int w, int h, int d, bool bitmap, Optimization optim )
 				w, h, data->d );
 
 #ifndef QT_NO_XFTFREETYPE
-    if ( qt_use_xrender && qt_has_xft ) {
+    if ( qt_has_xft ) {
 	if ( data->d == 1 ) {
 	    rendhd = (HANDLE) XftDrawCreateBitmap( x11Display(), hd );
 	} else {
@@ -408,7 +415,7 @@ QPixmap::QPixmap( int w, int h, const uchar *bits, bool isXbitmap)
 					(char *)bits, w, h );
 
 #ifndef QT_NO_XFTFREETYPE
-    if ( qt_use_xrender && qt_has_xft )
+    if ( qt_has_xft )
 	rendhd = (HANDLE) XftDrawCreateBitmap (x11Display (), hd);
 #endif // QT_NO_XFTFREETYPE
 
@@ -440,19 +447,18 @@ QPixmap::QPixmap( int w, int h, const uchar *bits, bool isXbitmap)
 
 void QPixmap::detach()
 {
-    if ( data->uninit || data->count == 1 )
-	data->uninit = FALSE;
-    else
-	*this = copy();
+    if ( data->count != 1 )
+        *this = copy();
+    data->uninit = FALSE;
 
-    // reset the cache data
+    // reset cached data
     if ( data->ximage ) {
-	qSafeXDestroyImage( (XImage*)data->ximage );
-	data->ximage = 0;
+        qSafeXDestroyImage( (XImage*)data->ximage );
+        data->ximage = 0;
     }
     if ( data->maskgc ) {
-	XFreeGC( x11Display(), (GC)data->maskgc );
-	data->maskgc = 0;
+        XFreeGC( x11Display(), (GC)data->maskgc );
+        data->maskgc = 0;
     }
 }
 
@@ -940,10 +946,10 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
     }
     detach();					// detach other references
     QImage  image = img;
-    int	 w   = image.width();
-    int	 h   = image.height();
+    const uint	 w   = image.width();
+    const uint	 h   = image.height();
     int	 d   = image.depth();
-    int	 dd  = x11Depth();
+    const int	 dd  = x11Depth();
     bool force_mono = (dd == 1 || isQBitmap() ||
 		       (conversion_flags & ColorMode_Mask)==MonoOnly );
 
@@ -1000,6 +1006,13 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 	    XFreePixmap( x11Display(), hd );
 	}
 
+	// make sure image.color(0) == color0 (white) and image.color(1) == color1 (black)
+	if (image.color(0) == Qt::black.rgb() && image.color(1) == Qt::white.rgb()) {
+	    image.invertPixels();
+	    image.setColor(0, Qt::white.rgb());
+	    image.setColor(1, Qt::black.rgb());
+	}
+
 	char  *bits;
 	uchar *tmp_bits;
 	int    bpl = (w+7)/8;
@@ -1009,7 +1022,7 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 	    Q_CHECK_PTR( tmp_bits );
 	    bits = (char *)tmp_bits;
 	    uchar *p, *b, *end;
-	    int y, count;
+	    uint y, count;
 	    if ( image.bitOrder() == QImage::BigEndian ) {
 		const uchar *f = qt_get_bitflip_array();
 		b = tmp_bits;
@@ -1045,7 +1058,7 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 					    bits, w, h );
 
 #ifndef QT_NO_XFTFREETYPE
-	if ( qt_use_xrender && qt_has_xft )
+	if ( qt_has_xft )
 	    rendhd = (HANDLE) XftDrawCreateBitmap( x11Display(), hd );
 #endif // QT_NO_XFTFREETYPE
 
@@ -1053,9 +1066,9 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 	    delete [] tmp_bits;
 	data->w = w;  data->h = h;  data->d = 1;
 
-	if ( img.hasAlphaBuffer() ) {
+	if ( image.hasAlphaBuffer() ) {
 	    QBitmap m;
-	    m = img.createAlphaMask( conversion_flags );
+	    m = image.createAlphaMask( conversion_flags );
 	    setMask( m );
 	}
 	return TRUE;
@@ -1067,32 +1080,31 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
     bool    trucol = (visual->c_class == TrueColor);
     int	    nbytes = image.numBytes();
     uchar  *newbits= 0;
-    register uchar *p;
 
     if ( trucol ) {				// truecolor display
 	QRgb  pix[256];				// pixel translation table
-	bool  d8 = d == 8;
-	uint  red_mask	  = (uint)visual->red_mask;
-	uint  green_mask  = (uint)visual->green_mask;
-	uint  blue_mask	  = (uint)visual->blue_mask;
-	int   red_shift	  = highest_bit( red_mask )   - 7;
-	int   green_shift = highest_bit( green_mask ) - 7;
-	int   blue_shift  = highest_bit( blue_mask )  - 7;
-	uint  rbits = highest_bit(red_mask) - lowest_bit(red_mask) + 1;
-	uint  gbits = highest_bit(green_mask) - lowest_bit(green_mask) + 1;
-	uint  bbits = highest_bit(blue_mask) - lowest_bit(blue_mask) + 1;
-	int   r, g, b;
+	const bool  d8 = d == 8;
+	const uint  red_mask	  = (uint)visual->red_mask;
+	const uint  green_mask  = (uint)visual->green_mask;
+	const uint  blue_mask	  = (uint)visual->blue_mask;
+	const int   red_shift	  = highest_bit( red_mask )   - 7;
+	const int   green_shift = highest_bit( green_mask ) - 7;
+	const int   blue_shift  = highest_bit( blue_mask )  - 7;
+	const uint  rbits = highest_bit(red_mask) - lowest_bit(red_mask) + 1;
+	const uint  gbits = highest_bit(green_mask) - lowest_bit(green_mask) + 1;
+	const uint  bbits = highest_bit(blue_mask) - lowest_bit(blue_mask) + 1;
 
 	if ( d8 ) {				// setup pixel translation
 	    QRgb *ctable = image.colorTable();
 	    for ( int i=0; i<image.numColors(); i++ ) {
-		r = qRed  (ctable[i]);
-		g = qGreen(ctable[i]);
-		b = qBlue (ctable[i]);
+		int r = qRed  (ctable[i]);
+		int g = qGreen(ctable[i]);
+		int b = qBlue (ctable[i]);
 		r = red_shift	> 0 ? r << red_shift   : r >> -red_shift;
 		g = green_shift > 0 ? g << green_shift : g >> -green_shift;
 		b = blue_shift	> 0 ? b << blue_shift  : b >> -blue_shift;
-		pix[i] = (b & blue_mask) | (g & green_mask) | (r & red_mask);
+		pix[i] = (b & blue_mask) | (g & green_mask) | (r & red_mask)
+			 | ~(blue_mask | green_mask | red_mask);
 	    }
 	}
 
@@ -1102,17 +1114,14 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 	Q_CHECK_PTR( newbits );
 	if ( !newbits )				// no memory
 	    return FALSE;
-	uchar *src;
-	uchar *dst;
-	QRgb   pixel;
 	int    bppc = xi->bits_per_pixel;
 
 	if ( bppc > 8 && xi->byte_order == LSBFirst )
 	    bppc++;
 
 	bool contig_bits = n_bits(red_mask) == rbits &&
-      n_bits(green_mask) == gbits &&
-       n_bits(blue_mask) == bbits;
+                           n_bits(green_mask) == gbits &&
+                           n_bits(blue_mask) == bbits;
 	bool dither_tc =
 	    // Want it?
 	    (conversion_flags & Dither_Mask) != ThresholdDither &&
@@ -1159,16 +1168,15 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 	    init=TRUE;
 	}
 
-	for ( int y=0; y<h; y++ ) {
-	    QRgb *p;
-	    src = image.scanLine( y );
-	    dst = newbits + xi->bytes_per_line*y;
-	    p	= (QRgb *)src;
+	for ( uint y=0; y<h; y++ ) {
+	    uchar* src = image.scanLine( y );
+	    uchar* dst = newbits + xi->bytes_per_line*y;
+	    QRgb* p = (QRgb *)src;
 
 #define GET_RGB \
-		r = qRed  ( *p ); \
-		g = qGreen( *p ); \
-		b = qBlue ( *p++ ); \
+		int r = qRed  ( *p ); \
+		int g = qGreen( *p ); \
+		int b = qBlue ( *p++ ); \
 		r = red_shift   > 0 \
 		    ? r << red_shift   : r >> -red_shift; \
 		g = green_shift > 0 \
@@ -1177,17 +1185,19 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 		    ? b << blue_shift  : b >> -blue_shift;
 
 #define GET_PIXEL \
+                int pixel; \
 		if ( d8 ) pixel = pix[*src++]; \
 		else { \
 		    GET_RGB \
-		    pixel = (b & blue_mask)|(g & green_mask) | (r & red_mask); \
+		    pixel = (b & blue_mask)|(g & green_mask) | (r & red_mask) \
+			    | ~(blue_mask | green_mask | red_mask); \
 		}
 
 #define GET_PIXEL_DITHER_TC \
-		r = qRed  ( *p ); \
-		g = qGreen( *p ); \
-		b = qBlue ( *p++ ); \
-		int thres = D[x%16][y%16]; \
+		int r = qRed  ( *p ); \
+		int g = qGreen( *p ); \
+		int b = qBlue ( *p++ ); \
+		const int thres = D[x%16][y%16]; \
 		if ( r <= (255-(1<<(8-rbits))) && ((r<<rbits) & 255) \
 			> thres) \
 		    r += (1<<(8-rbits)); \
@@ -1203,22 +1213,22 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 		    ? g << green_shift : g >> -green_shift; \
 		b = blue_shift  > 0 \
 		    ? b << blue_shift  : b >> -blue_shift; \
-		pixel = (b & blue_mask)|(g & green_mask) | (r & red_mask);
+		int pixel = (b & blue_mask)|(g & green_mask) | (r & red_mask);
 
-	    int x;
 	    if ( dither_tc ) {
+		uint x;
 		switch ( bppc ) {
 		case 16:			// 16 bit MSB
 		    for ( x=0; x<w; x++ ) {
 			GET_PIXEL_DITHER_TC
-			    *dst++ = (pixel >> 8);
+			*dst++ = (pixel >> 8);
 			*dst++ = pixel;
 		    }
 		    break;
 		case 17:			// 16 bit LSB
 		    for ( x=0; x<w; x++ ) {
 			GET_PIXEL_DITHER_TC
-			    *dst++ = pixel;
+			*dst++ = pixel;
 			*dst++ = pixel >> 8;
 		    }
 		    break;
@@ -1226,31 +1236,32 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 		    qFatal("Logic error");
 		}
 	    } else {
+		uint x;
 		switch ( bppc ) {
 		case 8:			// 8 bit
 		    for ( x=0; x<w; x++ ) {
-			pixel = pix[*src++];
+			int pixel = pix[*src++];
 			*dst++ = pixel;
 		    }
 		    break;
 		case 16:			// 16 bit MSB
 		    for ( x=0; x<w; x++ ) {
 			GET_PIXEL
-			    *dst++ = (pixel >> 8);
+			*dst++ = (pixel >> 8);
 			*dst++ = pixel;
 		    }
 		    break;
 		case 17:			// 16 bit LSB
 		    for ( x=0; x<w; x++ ) {
 			GET_PIXEL
-			    *dst++ = pixel;
+			*dst++ = pixel;
 			*dst++ = pixel >> 8;
 		    }
 		    break;
 		case 24:			// 24 bit MSB
 		    for ( x=0; x<w; x++ ) {
 			GET_PIXEL
-			    *dst++ = pixel >> 16;
+			*dst++ = pixel >> 16;
 			*dst++ = pixel >> 8;
 			*dst++ = pixel;
 		    }
@@ -1258,7 +1269,7 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 		case 25:			// 24 bit LSB
 		    for ( x=0; x<w; x++ ) {
 			GET_PIXEL
-			    *dst++ = pixel;
+			*dst++ = pixel;
 			*dst++ = pixel >> 8;
 			*dst++ = pixel >> 16;
 		    }
@@ -1266,7 +1277,7 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 		case 32:			// 32 bit MSB
 		    for ( x=0; x<w; x++ ) {
 			GET_PIXEL
-			    *dst++ = pixel >> 24;
+			*dst++ = pixel >> 24;
 			*dst++ = pixel >> 16;
 			*dst++ = pixel >> 8;
 			*dst++ = pixel;
@@ -1275,7 +1286,7 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 		case 33:			// 32 bit LSB
 		    for ( x=0; x<w; x++ ) {
 			GET_PIXEL
-			    *dst++ = pixel;
+			*dst++ = pixel;
 			*dst++ = pixel >> 8;
 			*dst++ = pixel >> 16;
 			*dst++ = pixel >> 24;
@@ -1290,15 +1301,15 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
     }
 
     if ( d == 8 && !trucol ) {			// 8 bit pixmap
-	int  i, j;
 	int  pop[256];				// pixel popularity
 
 	if ( image.numColors() == 0 )
 	    image.setNumColors( 1 );
 
 	memset( pop, 0, sizeof(int)*256 );	// reset popularity array
+	uint i;
 	for ( i=0; i<h; i++ ) {			// for each scanline...
-	    p = image.scanLine( i );
+	    uchar* p = image.scanLine( i );
 	    uchar *end = p + w;
 	    while ( p < end )			// compute popularity
 		pop[*p++]++;
@@ -1308,7 +1319,7 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 	Q_CHECK_PTR( newbits );
 	if ( !newbits )				// no memory
 	    return FALSE;
-	p = newbits;
+	uchar* p = newbits;
 	memcpy( p, image.bits(), nbytes );	// copy image data into newbits
 
 	/*
@@ -1323,7 +1334,7 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 	    int	  mindist;
 	};
 	int ncols = 0;
-	for ( i=0; i<image.numColors(); i++ ) { // compute number of colors
+	for ( i=0; i< (uint) image.numColors(); i++ ) { // compute number of colors
 	    if ( pop[i] > 0 )
 		ncols++;
 	}
@@ -1342,7 +1353,7 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 	int  maxpop = 0;
 	int  maxpix = 0;
 	Q_CHECK_PTR( pixarr );
-	j = 0;
+	uint j = 0;
 	QRgb* ctable = image.colorTable();
 	for ( i=0; i<256; i++ ) {		// init pixel array
 	    if ( pop[i] > 0 ) {
@@ -1364,7 +1375,7 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 	pixarr_sorted[0] = pixarr[maxpix];
 	pixarr[maxpix].use = 0;
 
-	for ( i=1; i<ncols; i++ ) {		// sort pixels
+	for ( i=1; i< (uint) ncols; i++ ) {		// sort pixels
 	    int minpix = -1, mindist = -1;
 	    px = &pixarr_sorted[i-1];
 	    int r = px->r;
@@ -1372,7 +1383,7 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 	    int b = px->b;
 	    int dist;
 	    if ( (i & 1) || i<10 ) {		// sort on max distance
-		for ( j=0; j<ncols; j++ ) {
+		for ( int j=0; j<ncols; j++ ) {
 		    px = &pixarr[j];
 		    if ( px->use ) {
 			dist = (px->r - r)*(px->r - r) +
@@ -1387,7 +1398,7 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 		    }
 		}
 	    } else {				// sort on max popularity
-		for ( j=0; j<ncols; j++ ) {
+		for ( int j=0; j<ncols; j++ ) {
 		    px = &pixarr[j];
 		    if ( px->use ) {
 			dist = (px->r - r)*(px->r - r) +
@@ -1408,14 +1419,14 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 
 	uint pix[256];				// pixel translation table
 	px = &pixarr_sorted[0];
-	for ( i=0; i<ncols; i++ ) {		// allocate colors
+	for ( i=0; i< (uint) ncols; i++ ) {		// allocate colors
 	    QColor c( px->r, px->g, px->b );
 	    pix[px->index] = c.pixel(x11Screen());
 	    px++;
 	}
 
 	p = newbits;
-	for ( i=0; i<nbytes; i++ ) {		// translate pixels
+	for ( i=0; i< (uint) nbytes; i++ ) {		// translate pixels
 	    *p = pix[*p];
 	    p++;
 	}
@@ -1430,10 +1441,10 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 	    Q_CHECK_PTR( newerbits );
 	    if ( !newerbits )				// no memory
 		return FALSE;
-	    p = newbits;
-	    for ( int y=0; y<h; y++ ) {		// OOPS: Do right byte order!!
+	    uchar* p = newbits;
+	    for ( uint y=0; y<h; y++ ) {		// OOPS: Do right byte order!!
 		p2 = newerbits + p2inc*y;
-		for ( int x=0; x<w; x++ )
+		for ( uint x=0; x<w; x++ )
 		    *p2++ = *p++;
 	    }
 	    free( newbits );
@@ -1447,7 +1458,7 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 	xi->data = (char *)newbits;
     }
 
-    if ( hd && (width() != w || height() != h || this->depth() != dd) ) {
+    if ( hd && (width() != (int)w || height() != (int)h || this->depth() != dd) ) {
 
 #ifndef QT_NO_XFTFREETYPE
 	if (rendhd) {
@@ -1465,7 +1476,7 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 				    w, h, dd );
 
 #ifndef QT_NO_XFTFREETYPE
-	if ( qt_use_xrender && qt_has_xft ) {
+	if ( qt_has_xft ) {
 	    if ( data->d == 1 ) {
 		rendhd = (HANDLE) XftDrawCreateBitmap( x11Display (), hd );
 	    } else {
@@ -1490,14 +1501,26 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
     data->h = h;
     data->d = dd;
 
-    if ( img.hasAlphaBuffer() ) {
+    if ( image.hasAlphaBuffer() ) {
 	QBitmap m;
-	m = img.createAlphaMask( conversion_flags );
+	m = image.createAlphaMask( conversion_flags );
 	setMask( m );
 
 #ifndef QT_NO_XFTFREETYPE
-	// ### only support 32bit images at the moment
-	if (qt_use_xrender && qt_has_xft && img.depth() == 32) {
+	// does this image have an alphamap (and not just a 1bpp mask)?
+	bool alphamap = image.depth() == 32;
+	if (image.depth() == 8) {
+	    const QRgb * const rgb = image.colorTable();
+	    for (int i = 0, count = image.numColors(); i < count; ++i) {
+		const int alpha = qAlpha(rgb[i]);
+		if (alpha != 0 && alpha != 0xff) {
+		    alphamap = TRUE;
+		    break;
+		}
+	    }
+	}
+
+	if (qt_use_xrender && qt_has_xft && alphamap) {
 	    data->alphapm = new QPixmap; // create a null pixmap
 
 	    // setup pixmap data
@@ -1520,12 +1543,21 @@ bool QPixmap::convertFromImage( const QImage &img, int conversion_flags )
 		// the data is deleted by qSafeXDestroyImage
 		axi->data = (char *) malloc(h * axi->bytes_per_line);
 		Q_CHECK_PTR( axi->data );
-
 		char *aptr = axi->data;
-		int *iptr = (int *) image.bits();
-		int max = w * h;
-		for (int i = 0; i < max; i++)
-		    *aptr++ = *iptr++ >> 24; // squirt
+
+		if (image.depth() == 32) {
+		    const int *iptr = (const int *) image.bits();
+		    int max = w * h;
+		    while (max--)
+			*aptr++ = *iptr++ >> 24; // squirt
+		} else if (image.depth() == 8) {
+		    const QRgb * const rgb = image.colorTable();
+		    for (uint y = 0; y < h; ++y) {
+			const uchar *iptr = image.scanLine(y);
+			for (uint x = 0; x < w; ++x)
+			    *aptr++ = qAlpha(rgb[*iptr++]);
+		    }
+		}
 
 		GC gc = XCreateGC(x11Display(), data->alphapm->hd, 0, 0);
 		XPutImage(dpy, data->alphapm->hd, gc, axi, 0, 0, 0, 0, w, h);
