@@ -1,11 +1,11 @@
 /**********************************************************************
-** $Id: qt/qprinter.cpp   3.2.3   edited Jul 16 12:54 $
+** $Id: qt/qprinter.cpp   3.3.5   edited Aug 31 12:13 $
 **
 ** Implementation of QPrinter class
 **
 ** Created : 941003
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -113,9 +113,10 @@
     Once you start printing, calling newPage() is essential. You will
     probably also need to look at the QPaintDeviceMetrics for the
     printer (see the \link simple-application.html#printersimple print
-    function\endlink in the Application walk-through). Note that the
-    paint device metrics are valid only after the QPrinter has been
-    set up, i.e. after setup() has returned successfully.
+    function\endlink in the Application walk-through). In previous versions,
+    paint device metrics were valid only after the QPrinter has been set
+    up, i.e. after setup() has returned successfully. This is no longer
+    the case and paint device metrics can be requested safely before set up.
 
     If you want to abort the print job, abort() will try its best to
     stop printing. It may cancel the entire job or just some of it.
@@ -124,8 +125,8 @@
     \endomit
 
     If your current locale converts "," to ".", you will need to set
-    a locale that doen't do this (e.g. the "C" locale) before using
-    QPrinter.
+    a locale (via the standard C setlocale() function) that doen't do this
+    before using QPrinter. The "C" locale works well for this.
 
     The TrueType font embedding for Qt's postscript driver uses code
     by David Chappell of Trinity College Computing Center.
@@ -212,8 +213,8 @@
 
     The defined sizes (with setFullPage(TRUE)) are:
 
-    \value A0 841 x 1189 mm
-    \value A1 594 x 841 mm
+    \value A0 841 x 1189 mm This value is not supported on windows.
+    \value A1 594 x 841 mm This value is not supported on windows.
     \value A2 420 x 594 mm
     \value A3 297 x 420 mm
     \value A4 210 x 297 mm, 8.26 x 11.7 inches
@@ -346,7 +347,7 @@
     if the output should be sent directly to the printer. The default
     setting is FALSE.
 
-    This function is currently only supported under X11.
+    This function is currently only supported under X11 and Mac OS X.
 
     \sa setOutputToFile(), setOutputFileName()
 */
@@ -358,7 +359,7 @@
     Will output to a file if \a enable is TRUE, or will output
     directly to the printer if \a enable is FALSE.
 
-    This function is currently only supported under X11.
+    This function is currently only supported under X11 and Mac OS X.
 
     \sa outputToFile(), setOutputFileName()
 */
@@ -569,6 +570,9 @@ void QPrinter::setPageSize( PageSize newPageSize )
 
     This function is useful mostly for setting a default value that
     the user can override in the print dialog when you call setup().
+
+    \bug This value is not kept in sync with the Windows or Mac OS X printer
+    dialogs.
 */
 
 void QPrinter::setPageOrder( PageOrder newPageOrder )
@@ -584,6 +588,9 @@ void QPrinter::setPageOrder( PageOrder newPageOrder )
     Returns the current page order.
 
     The default page order is \c FirstPageFirst.
+
+    \bug This value is not kept in sync with the Windows or Mac OS X printer
+    dialogs.
 */
 
 QPrinter::PageOrder QPrinter::pageOrder() const
@@ -724,11 +731,11 @@ void QPrinter::setMinMax( int minPage, int maxPage )
 
     Returns the number of copies to be printed. The default value is 1.
 
-    After a call to setup(), this value will return the number of
-    times the application is required to print in order to match the
-    number specified in the printer setup dialog. This has been done since
-    some printer drivers are not capable of buffering up the copies and
-    the application in those cases have to make an explicit call to the
+    This value will return the number of times the application is
+    required to print in order to match the number specified in the
+    printer setup dialog. This has been done since some printer
+    drivers are not capable of buffering up the copies and the
+    application in those cases have to make an explicit call to the
     print code for each copy.
 
     \sa setNumCopies()
@@ -801,7 +808,7 @@ void QPrinter::setCollateCopies(bool on)
 }
 
 /*!
-    Sets the number of pages to be printed to \a numCopies.
+    Sets the number of copies to be printed to \a numCopies.
 
     The printer driver reads this setting and prints the specified
     number of copies.
