@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qprinter_unix.cpp   3.3.3   edited Jul 30 15:56 $
+** $Id: qt/qprinter_unix.cpp   3.3.5   edited Aug 31 12:13 $
 **
 ** Implementation of QPrinter class for Unix
 **
 ** Created : 950810
 **
-** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -138,7 +138,7 @@ QPrinter::QPrinter( PrinterMode m )
     }
 
     d = new QPrinterUnixPrivate;
-    D->marginsSpecified = false;
+    D->marginsSpecified = FALSE;
     d->printerOptions = 0;
     setOptionEnabled( PrintToFile, TRUE );
     setOptionEnabled( PrintPageRange, TRUE );
@@ -413,6 +413,7 @@ bool QPrinter::cmd( int c, QPainter *paint, QPDevCmdParam *p )
                         uint i;
                         for (i = 0; i < lphack.size(); ++i)
                             lpargs[i+1] = (char *)lphack[i].ascii();
+#ifndef Q_OS_OSF
                         if (psToStr[page_size]) {
                             lpargs[++i] = "-o";
                             lpargs[++i] = (char *)psToStr[page_size];
@@ -421,6 +422,7 @@ bool QPrinter::cmd( int c, QPainter *paint, QPDevCmdParam *p )
                             media += psToStr[page_size];
                             lpargs[++i] = (char *)media.ascii();
                         }
+#endif
                         lpargs[++i] = 0;
                         char **lprargs = new char *[lprhack.size()+1];
                         lprargs[0] = "lpr";
