@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qvariant.cpp   3.2.3   edited Sep 22 09:17 $
+** $Id: qt/qvariant.cpp   3.3.5   edited Aug 31 12:13 $
 **
 ** Implementation of QVariant class
 **
 ** Created : 990414
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -71,6 +71,14 @@
 // #define QVARIANT_DEBUG
 
 
+static bool isNumeric(QVariant::Type type)
+{
+    return (type == QVariant::Int || type == QVariant::UInt
+	    || type == QVariant::Double || type == QVariant::LongLong
+	    || type == QVariant::ULongLong || type == QVariant::Bool);
+}
+
+
 #ifdef QVARIANT_DEBUG
 int qv_count = 0;
 int get_qv_count() { return qv_count; }
@@ -196,6 +204,12 @@ QVariant::Private::Private( Private* d )
 	case QVariant::UInt:
 	    value.u = d->value.u;
 	    break;
+	case QVariant::LongLong:
+	    value.ll = d->value.ll;
+	    break;
+	case QVariant::ULongLong:
+	    value.ull = d->value.ull;
+	    break;
 	case QVariant::Bool:
 	    value.b = d->value.b;
 	    break;
@@ -1028,7 +1042,7 @@ const char* QVariant::typeToName( Type typ )
 
 
 /*!
-    Converts the string representation of the storage type gven in \a
+    Converts the string representation of the storage type given in \a
     name, to its enum representation.
 
     If the string representation cannot be converted to any enum
@@ -1052,6 +1066,7 @@ QVariant::Type QVariant::nameToType( const char* name )
 */
 void QVariant::load( QDataStream& s )
 {
+    clear();
     Q_UINT32 u;
     s >> u;
     Type t = (Type)u;
@@ -1606,11 +1621,10 @@ QDataStream& operator<< ( QDataStream& s, const QVariant::Type p )
 */
 
 /*!
-    Returns the variant as a QString if the variant has type() String,
-    CString, ByteArray, Int, Uint, Bool, Double, Date, Time, DateTime,
-    KeySequence, Font or Color; otherwise returns QString::null.
+    Returns the variant as a QString if the variant can be cast to
+    String, otherwise returns QString::null.
 
-    \sa asString()
+    \sa asString(), canCast()
 */
 const QString QVariant::toString() const
 {
@@ -1654,20 +1668,22 @@ const QString QVariant::toString() const
     }
 }
 /*!
-    Returns the variant as a QCString if the variant has type()
-    CString or String; otherwise returns 0.
+    Returns the variant as a QCString if the variant can be cast to a
+    CString; otherwise returns 0.
 
-    \sa asCString()
+    \sa asCString(), canCast()
 */
 const QCString QVariant::toCString() const
 {
-    switch ( d->typ ) {
-    case CString:
-	return *((QCString*)d->value.ptr);
-    case String:
-	return ((QString*)d->value.ptr)->latin1();
-    default:
-	return 0;
+    switch( d->typ ) {
+    case CString: return *((QCString*)d->value.ptr);
+    case String: return ((QString*)d->value.ptr)->latin1();
+    default: {
+	if (!canCast(String))
+	    return 0;
+	QString c = toString();
+	return QCString(c.latin1());
+    }
     }
 }
 
@@ -1743,14 +1759,16 @@ const QMap<QString, QVariant> QVariant::toMap() const
 }
 #endif
 /*!
-    Returns the variant as a QFont if the variant has type() Font;
+    Returns the variant as a QFont if the variant can be cast to Font;
     otherwise returns the application's default font.
 
-  \sa asFont()
+  \sa asFont(), canCast()
 */
 const QFont QVariant::toFont() const
 {
     switch ( d->typ ) {
+    case CString:
+    case ByteArray:
     case String:
 	{
 	    QFont fnt;
@@ -1849,20 +1867,22 @@ const QSize QVariant::toSize() const
 }
 
 /*!
-    Returns the variant as a QColor if the variant has type() Color;
+    Returns the variant as a QColor if the variant can be cast to Color;
     otherwise returns an invalid color.
 
-    \sa asColor()
+    \sa asColor(), canCast()
 */
 const QColor QVariant::toColor() const
 {
     switch ( d->typ ) {
+    case ByteArray:
+    case CString:
     case String:
-	{
-	    QColor col;
-	    col.setNamedColor( toString() );
-	    return col;
-	}
+    {
+	QColor col;
+	col.setNamedColor( toString() );
+	return col;
+    }
     case Color:
 	return *((QColor*)d->value.ptr);
     default:
@@ -1972,13 +1992,14 @@ const QCursor QVariant::toCursor() const
 }
 
 /*!
-    Returns the variant as a QDate if the variant has type() Date,
-    DateTime or String; otherwise returns an invalid date.
+    Returns the variant as a QDate if the variant can be cast to Date;
+    otherwise returns an invalid date.
 
-    Note that if the type() is String an invalid date will be returned
-    if the string cannot be parsed as a Qt::ISODate format date.
+    Note that if the type() is String, CString or ByteArray an invalid
+    date will be returned if the string cannot be parsed as a
+    Qt::ISODate format date.
 
-    \sa asDate()
+    \sa asDate(), canCast()
 */
 const QDate QVariant::toDate() const
 {
@@ -1990,6 +2011,9 @@ const QDate QVariant::toDate() const
 #ifndef QT_NO_DATESTRING
     case String:
 	return QDate::fromString( *((QString*)d->value.ptr), Qt::ISODate );
+    case CString:
+    case ByteArray:
+	return QDate::fromString(toString(), Qt::ISODate);
 #endif
     default:
 	return QDate();
@@ -1997,11 +2021,12 @@ const QDate QVariant::toDate() const
 }
 
 /*!
-    Returns the variant as a QTime if the variant has type() Time,
-    DateTime or String; otherwise returns an invalid time.
+    Returns the variant as a QTime if the variant can be cast to Time;
+    otherwise returns an invalid date.
 
-    Note that if the type() is String an invalid time will be returned
-    if the string cannot be parsed as a Qt::ISODate format time.
+    Note that if the type() is String, CString or ByteArray an invalid
+    time will be returned if the string cannot be parsed as a
+    Qt::ISODate format time.
 
     \sa asTime()
 */
@@ -2015,6 +2040,9 @@ const QTime QVariant::toTime() const
 #ifndef QT_NO_DATESTRING
     case String:
 	return QTime::fromString( *((QString*)d->value.ptr), Qt::ISODate );
+    case CString:
+    case ByteArray:
+	return QTime::fromString(toString(), Qt::ISODate);
 #endif
     default:
 	return QTime();
@@ -2022,12 +2050,12 @@ const QTime QVariant::toTime() const
 }
 
 /*!
-    Returns the variant as a QDateTime if the variant has type()
-    DateTime, Date or String; otherwise returns an invalid date/time.
+    Returns the variant as a QDateTime if the variant can be cast to
+    DateTime; otherwise returns an invalid QDateTime.
 
-    Note that if the type() is String an invalid date/time will be
-    returned if the string cannot be parsed as a Qt::ISODate format
-    date/time.
+    Note that if the type() is String, CString or ByteArray an invalid
+    QDateTime will be returned if the string cannot be parsed as a
+    Qt::ISODate format date/time.
 
     \sa asDateTime()
 */
@@ -2039,6 +2067,9 @@ const QDateTime QVariant::toDateTime() const
 #ifndef QT_NO_DATESTRING
     case String:
 	return QDateTime::fromString( *((QString*)d->value.ptr), Qt::ISODate );
+    case CString:
+    case ByteArray:
+	return QDateTime::fromString(toString(), Qt::ISODate);
 #endif
     case Date:
 	return QDateTime( *((QDate*)d->value.ptr) );
@@ -2048,18 +2079,25 @@ const QDateTime QVariant::toDateTime() const
 }
 
 /*!
-    Returns the variant as a QByteArray if the variant has type()
-    ByteArray or CString; otherwise returns an empty bytearray.
+    Returns the variant as a QByteArray if the variant can be cast to
+    a ByteArray; otherwise returns an empty bytearray.
 
-    \sa asByteArray()
+    \sa asByteArray(), canCast()
 */
 const QByteArray QVariant::toByteArray() const
 {
-    if ( d->typ == ByteArray )
-	return *((QByteArray*)d->value.ptr);
-    if ( d->typ == CString )
-	return *((QByteArray*)d->value.ptr);
-    return QByteArray();
+    switch(d->typ) {
+    case ByteArray: return *((QByteArray*)d->value.ptr);
+    case CString: return *((QByteArray*)d->value.ptr);
+    default: {
+	QByteArray ret;
+	if (canCast(String)) {
+	    QString c = toString();
+	    ret.duplicate(c.latin1(), c.length());
+	}
+	return ret;
+    }
+    }
 }
 
 /*!
@@ -2078,14 +2116,10 @@ const QBitArray QVariant::toBitArray() const
 #ifndef QT_NO_ACCEL
 
 /*!
-    Returns the variant as a QKeySequence if the variant has type()
-    KeySequence, Int or String; otherwise returns an empty key
-    sequence.
+    Returns the variant as a QKeySequence if the variant can be cast
+    to a KeySequence; otherwise returns an empty key sequence.
 
-    Note that not all Ints and Strings are valid key sequences and in
-    such cases an empty key sequence will be returned.
-
-    \sa asKeySequence()
+    \sa asKeySequence(), canCast()
 */
 const QKeySequence QVariant::toKeySequence() const
 {
@@ -2093,8 +2127,14 @@ const QKeySequence QVariant::toKeySequence() const
     case KeySequence:
 	return *((QKeySequence*)d->value.ptr);
     case String:
+    case ByteArray:
+    case CString:
 	return QKeySequence( toString() );
     case Int:
+    case UInt:
+    case Double:
+    case ULongLong:
+    case LongLong:
 	return QKeySequence( toInt() );
     default:
 	return QKeySequence();
@@ -2118,14 +2158,13 @@ const QPen QVariant::toPen() const
 }
 
 /*!
-    Returns the variant as an int if the variant has type() String,
-    CString, Int, UInt, Double, Bool or KeySequence; otherwise returns
-    0.
+    Returns the variant as an int if the variant can be cast to Int;
+    otherwise returns 0.
 
     If \a ok is non-null: \a *ok is set to TRUE if the value could be
     converted to an int; otherwise \a *ok is set to FALSE.
 
-    \sa asInt() canCast()
+    \sa asInt(), canCast()
 */
 int QVariant::toInt( bool * ok ) const
 {
@@ -2136,6 +2175,7 @@ int QVariant::toInt( bool * ok ) const
     case String:
 	return ((QString*)d->value.ptr)->toInt( ok );
     case CString:
+    case ByteArray:
 	return ((QCString*)d->value.ptr)->toInt( ok );
     case Int:
 	return d->value.i;
@@ -2159,13 +2199,13 @@ int QVariant::toInt( bool * ok ) const
 }
 
 /*!
-    Returns the variant as an unsigned int if the variant has type()
-    String, CString, UInt, Int, Double, or Bool; otherwise returns 0.
+    Returns the variant as an unsigned int if the variant can be cast
+    to UInt; otherwise returns 0.
 
     If \a ok is non-null: \a *ok is set to TRUE if the value could be
     converted to an unsigned int; otherwise \a *ok is set to FALSE.
 
-    \sa asUInt()
+    \sa asUInt(), canCast()
 */
 uint QVariant::toUInt( bool * ok ) const
 {
@@ -2176,6 +2216,7 @@ uint QVariant::toUInt( bool * ok ) const
     case String:
 	return ((QString*)d->value.ptr)->toUInt( ok );
     case CString:
+    case ByteArray:
 	return ((QCString*)d->value.ptr)->toUInt( ok );
     case Int:
 	return (uint)d->value.i;
@@ -2195,14 +2236,13 @@ uint QVariant::toUInt( bool * ok ) const
 }
 
 /*!
-    Returns the variant as a long long int if the variant has type()
-    LongLong, ULongLong, any type allowing a toInt() conversion;
-    otherwise returns 0.
+    Returns the variant as a long long int if the variant can be cast
+    to LongLong; otherwise returns 0.
 
     If \a ok is non-null: \a *ok is set to TRUE if the value could be
     converted to an int; otherwise \a *ok is set to FALSE.
 
-    \sa asLongLong() canCast()
+    \sa asLongLong(), canCast()
 */
 Q_LLONG QVariant::toLongLong( bool * ok ) const
 {
@@ -2213,7 +2253,8 @@ Q_LLONG QVariant::toLongLong( bool * ok ) const
     case String:
 	return ((QString*)d->value.ptr)->toLongLong( ok );
     case CString:
-	return ((QCString*)d->value.ptr)->toInt( ok ); // ### LongLong
+    case ByteArray:
+	return QString(*(QCString*)d->value.ptr).toLongLong(ok);
     case Int:
 	return (Q_LLONG)d->value.i;
     case UInt:
@@ -2233,13 +2274,12 @@ Q_LLONG QVariant::toLongLong( bool * ok ) const
 
 /*!
     Returns the variant as as an unsigned long long int if the variant
-    has type() LongLong, ULongLong, any type allowing a toUInt()
-    conversion; otherwise returns 0.
+    can be cast to ULongLong; otherwise returns 0.
 
     If \a ok is non-null: \a *ok is set to TRUE if the value could be
     converted to an int; otherwise \a *ok is set to FALSE.
 
-    \sa asULongLong() canCast()
+    \sa asULongLong(), canCast()
 */
 Q_ULLONG QVariant::toULongLong( bool * ok ) const
 {
@@ -2262,20 +2302,23 @@ Q_ULLONG QVariant::toULongLong( bool * ok ) const
     case String:
 	return ((QString*)d->value.ptr)->toULongLong( ok );
     case CString:
-	// ###	return ((QCString*)d->value.ptr)->toULongLong( ok );
+    case ByteArray:
+	return QString(*(QCString*)d->value.ptr).toULongLong(ok);
     default:
 	return 0;
     }
 }
 
 /*!
-    Returns the variant as a bool if the variant has type() Bool.
+    Returns the variant as a bool if the variant can be cast to Bool;
+    otherWise returns FALSE.
 
-    Returns TRUE if the variant has type Int, UInt or Double and its
-    value is non-zero, or if the variant has type String and its lower-case
-    content is not empty, "0" or "false"; otherwise returns FALSE.
+    Returns TRUE if the variant has a numeric type and its value is
+    non-zero, or if the variant has type String, ByteArray or CString
+    and its lower-case content is not empty, "0" or "false"; otherwise
+    returns FALSE.
 
-    \sa asBool()
+    \sa asBool(), canCast()
 */
 bool QVariant::toBool() const
 {
@@ -2293,6 +2336,8 @@ bool QVariant::toBool() const
     case ULongLong:
 	return d->value.ull != 0;
     case String:
+    case CString:
+    case ByteArray:
 	{
 	    QString str = toString().lower();
 	    return !(str == "0" || str == "false" || str.isEmpty() );
@@ -2303,14 +2348,13 @@ bool QVariant::toBool() const
 }
 
 /*!
-    Returns the variant as a double if the variant has type() String,
-    CString, Double, Int, UInt, LongLong, ULongLong or Bool; otherwise
-    returns 0.0.
+    Returns the variant as a double if the variant can be cast to
+    Double; otherwise returns 0.0.
 
     If \a ok is non-null: \a *ok is set to TRUE if the value could be
     converted to a double; otherwise \a *ok is set to FALSE.
 
-    \sa asDouble()
+    \sa asDouble(), canCast()
 */
 double QVariant::toDouble( bool * ok ) const
 {
@@ -2321,6 +2365,7 @@ double QVariant::toDouble( bool * ok ) const
     case String:
 	return ((QString*)d->value.ptr)->toDouble( ok );
     case CString:
+    case ByteArray:
 	return ((QCString*)d->value.ptr)->toDouble( ok );
     case Double:
 	return d->value.d;
@@ -2394,8 +2439,16 @@ QSizePolicy QVariant::toSizePolicy() const
 }
 
 
-#define Q_VARIANT_AS( f ) Q##f& QVariant::as##f() { \
-   if ( d->typ != f ) *this = QVariant( to##f() ); else detach(); return *((Q##f*)d->value.ptr);}
+#define Q_VARIANT_AS( f ) Q##f& QVariant::as##f() \
+{ \
+    bool b = isNull(); \
+    if ( d->typ != f ) \
+	*this = QVariant( to##f() ); \
+    else \
+	detach(); \
+    d->is_null = b; \
+    return *((Q##f*)d->value.ptr); \
+}
 
 Q_VARIANT_AS(String)
 Q_VARIANT_AS(CString)
@@ -2731,12 +2784,14 @@ Q_VARIANT_AS(Pen)
 */
 int& QVariant::asInt()
 {
+    detach();
     if ( d->typ != Int ) {
-	detach();
 	int i = toInt();
+	bool b = isNull();
 	d->clear();
  	d->value.i = i;
 	d->typ = Int;
+	d->is_null = b;
     }
     return d->value.i;
 }
@@ -2746,12 +2801,14 @@ int& QVariant::asInt()
 */
 uint& QVariant::asUInt()
 {
+    detach();
     if ( d->typ != UInt ) {
-	detach();
 	uint u = toUInt();
+	bool b = isNull();
 	d->clear();
 	d->value.u = u;
 	d->typ = UInt;
+	d->is_null = b;
     }
     return d->value.u;
 }
@@ -2761,12 +2818,14 @@ uint& QVariant::asUInt()
 */
 Q_LLONG& QVariant::asLongLong()
 {
+    detach();
     if ( d->typ != LongLong ) {
-	detach();
 	Q_LLONG ll = toLongLong();
+	bool b = isNull();
 	d->clear();
  	d->value.ll = ll;
 	d->typ = LongLong;
+	d->is_null = b;
     }
     return d->value.ll;
 }
@@ -2776,12 +2835,14 @@ Q_LLONG& QVariant::asLongLong()
 */
 Q_ULLONG& QVariant::asULongLong()
 {
+    detach();
     if ( d->typ != ULongLong ) {
-	detach();
 	Q_ULLONG ull = toULongLong();
+	bool b = isNull();
 	d->clear();
  	d->value.ull = ull;
 	d->typ = ULongLong;
+	d->is_null = b;
     }
     return d->value.ull;
 }
@@ -2791,12 +2852,14 @@ Q_ULLONG& QVariant::asULongLong()
 */
 bool& QVariant::asBool()
 {
+    detach();
     if ( d->typ != Bool ) {
-	detach();
 	bool b = toBool();
+	bool nb = isNull();
 	d->clear();
 	d->value.b = b;
 	d->typ = Bool;
+	d->is_null = nb;
     }
     return d->value.b;
 }
@@ -2806,12 +2869,14 @@ bool& QVariant::asBool()
 */
 double& QVariant::asDouble()
 {
+    detach();
     if ( d->typ != Double ) {
-	detach();
 	double dbl = toDouble();
+	bool b = isNull();
 	d->clear();
 	d->value.d = dbl;
 	d->typ = Double;
+	d->is_null = b;
     }
     return d->value.d;
 }
@@ -2833,8 +2898,12 @@ double& QVariant::asDouble()
 */
 QValueList<QVariant>& QVariant::asList()
 {
+    bool b = isNull();
     if ( d->typ != List )
 	*this = QVariant( toList() );
+    else
+	detach();
+    d->is_null = b;
     return *((QValueList<QVariant>*)d->value.ptr);
 }
 
@@ -2854,8 +2923,12 @@ QValueList<QVariant>& QVariant::asList()
 */
 QMap<QString, QVariant>& QVariant::asMap()
 {
+    bool b = isNull();
     if ( d->typ != Map )
 	*this = QVariant( toMap() );
+    else
+	detach();
+    d->is_null = b;
     return *((QMap<QString,QVariant>*)d->value.ptr);
 }
 #endif
@@ -2868,66 +2941,123 @@ QMap<QString, QVariant>& QVariant::asMap()
     The following casts are done automatically:
     \table
     \header \i Type \i Automatically Cast To
-    \row \i Bool \i Double, Int, UInt, LongLong, ULongLong
-    \row \i Color \i String
-    \row \i Date \i String, DateTime
-    \row \i DateTime \i String, Date, Time
-    \row \i Double \i String, Int, Bool, UInt
-    \row \i Font \i String
-    \row \i Int \i String, Double, Bool, UInt
-    \row \i List \i StringList (if the list contains strings or
+    \row \i Bool \i Double, Int, UInt, LongLong, ULongLong, String, CString, ByteArray
+    \row \i Color \i String. CString. ByteArray
+    \row \i Date \i String, CString, ByteArray, DateTime
+    \row \i DateTime \i String, CString, ByteArray, Date, Time
+    \row \i Double \i String, CString, ByteArray, Int, Bool, UInt, LongLong, ULongLong
+    \row \i Font \i String, CString, ByteArray
+    \row \i Int \i String, CString, ByteArray, Double, Bool, UInt, LongLong, ULongLong, KeySequence
+    \row \i LongLong \i String, CString, ByteArray, Double, Bool, UInt, LongLong, ULongLong, KeySequence
+    \row \i ULongLong \i String, CString, ByteArray, Double, Bool, UInt, LongLong, ULongLong, KeySequence
+    \row \i List \i StringList (if the list contains only strings or
     something that can be cast to a string)
-    \row \i String \i CString, Int, Uint, Bool, Double, Date,
+    \row \i String \i CString, ByteArray, CString, Int, UInt, Bool, Double, Date,
     Time, DateTime, KeySequence, Font, Color
-    \row \i CString \i String
+    \row \i CString \i String, ByteArray, Int, UInt, Bool, Double, Date, ULongLong, LongLong
+    \row \i ByteArray \i String, CString, Int, UInt, Bool, Double, Date, ULongLong, LongLong
     \row \i StringList \i List
     \row \i Time \i String
-    \row \i UInt \i String, Double, Bool, Int
-    \row \i KeySequence \i String, Int
+    \row \i Int \i String, CString, ByteArray, Double, Bool, UInt, LongLong, ULongLong, KeySequence
+    \row \i KeySequence \i String, CString, ByteArray, Int, UInt, LongLong, ULongLong
     \endtable
 */
 bool QVariant::canCast( Type t ) const
 {
     if ( Type( d->typ ) == t )
 	return TRUE;
+
     switch ( t ) {
     case Bool:
-	return d->typ == Double || d->typ == Int || d->typ == UInt ||
-	       d->typ == LongLong || d->typ == ULongLong || d->typ == String;
-    case Int:
-	return d->typ == String || d->typ == Double || d->typ == Bool ||
-	       d->typ == UInt || d->typ == LongLong || d->typ == ULongLong ||
-	       d->typ == KeySequence;
-    case UInt:
-	return d->typ == String || d->typ == Double || d->typ == Bool ||
-	       d->typ == Int || d->typ == LongLong || d->typ == ULongLong;
-    case LongLong:
-	return d->typ == String || d->typ == Double || d->typ == Bool ||
-	       d->typ == Int || d->typ == UInt || d->typ == ULongLong ||
-	       d->typ == KeySequence;
-    case ULongLong:
-	return d->typ == String || d->typ == Double || d->typ == Bool ||
-	       d->typ == Int || d->typ == UInt || d->typ == LongLong;
     case Double:
-	return d->typ == String || d->typ == Int || d->typ == Bool || d->typ == UInt;
+	if (d->typ == KeySequence)
+	    break;
+    case Int:
+    case UInt:
+    case LongLong:
+    case ULongLong:
+	switch(d->typ) {
+	case Bool:
+	case ByteArray:
+	case CString:
+	case Double:
+	case Int:
+	case KeySequence:
+	case LongLong:
+	case String:
+	case UInt:
+	case ULongLong:
+	    return TRUE;
+	default: break;
+	}
+	break;
+
     case CString:
-	return d->typ == String;
-    case String:
-	return d->typ == CString || d->typ == ByteArray || d->typ == Int || d->typ == UInt || d->typ == Bool || d->typ == Double || d->typ == Date || d->typ == Time || d->typ == DateTime || d->typ == KeySequence || d->typ == Font || d->typ == Color || d->typ == LongLong || d->typ == ULongLong;
     case ByteArray:
-	return d->typ == CString;
-    case Date:
-	return d->typ == String || d->typ == DateTime;
+    case String:
+	switch(d->typ) {
+	case Bool:
+	case ByteArray:
+	case CString:
+	case Color:
+	case Date:
+	case DateTime:
+	case Double:
+	case Font:
+	case Int:
+	case KeySequence:
+	case LongLong:
+	case String:
+	case Time:
+	case UInt:
+	case ULongLong:
+	    return TRUE;
+	default: break;
+	}
+	break;
+
     case Time:
-	return d->typ == String || d->typ == DateTime;
+	if (d->typ == Date)
+	    break;
+    case Date:
     case DateTime:
-	return d->typ == String || d->typ == Date;
+	switch(d->typ) {
+	case ByteArray:
+	case CString:
+	case Date:
+	case DateTime:
+	case String:
+	    return TRUE;
+	default: break;
+	}
+	break;
+
     case KeySequence:
-	return d->typ == String || d->typ == Int;
+	switch(d->typ) {
+	case ByteArray:
+	case CString:
+	case Int:
+	case UInt:
+	case LongLong:
+	case ULongLong:
+	case Double:
+	case String:
+	    return TRUE;
+	default: break;
+	}
+	break;
+
     case Font:
-	return d->typ == String;
     case Color:
-	return d->typ == String;
+	switch(d->typ) {
+	case ByteArray:
+	case CString:
+	case String:
+	    return TRUE;
+	default: break;
+	}
+	break;
+
 #ifndef QT_NO_STRINGLIST
     case List:
 	return d->typ == StringList;
@@ -2943,11 +3073,28 @@ bool QVariant::canCast( Type t ) const
 	    }
 	    return TRUE;
 	}
-	return FALSE;
 #endif
-    default:
-	return FALSE;
+    case Invalid:
+    case Map:
+    case Pixmap:
+    case Brush:
+    case Rect:
+    case Size:
+    case Palette:
+    case ColorGroup:
+    case IconSet:
+    case Point:
+    case Image:
+    case PointArray:
+    case Region:
+    case Bitmap:
+    case Cursor:
+    case SizePolicy:
+    case BitArray:
+    case Pen:
+	break;
     }
+    return FALSE;
 }
 
 /*!
@@ -3090,8 +3237,49 @@ bool QVariant::cast( Type t )
 
 bool QVariant::operator==( const QVariant &v ) const
 {
-    if ( !v.canCast( type() ) )
+    if (isNumeric(v.type()) && canCast(v.type())) {
+	bool ok;
+	switch(v.type()) {
+	case Bool:
+	    return toBool() == v.toBool();
+	case Int:
+	{
+	    int val = toInt(&ok);
+	    return (ok && val == v.toInt());
+	}
+	case UInt:
+	{
+	    uint val = toUInt(&ok);
+	    return (ok && val == v.toUInt());
+	}
+
+	case Double:
+	{
+	    double val = toDouble(&ok);
+	    return (ok && val == v.toDouble());
+	}
+
+	case LongLong:
+	{
+	    Q_LLONG val = toLongLong(&ok);
+	    return (ok && val == v.toLongLong());
+	}
+
+	case ULongLong:
+	{
+	    Q_ULLONG val = toULongLong(&ok);
+	    return (ok && val == v.toULongLong());
+	}
+
+	default:
+	    Q_ASSERT(FALSE);
+	}
+    }
+
+    if (!v.canCast(d->typ)) {
 	return FALSE;
+    }
+
     switch( d->typ ) {
     case Cursor:
 #ifndef QT_NO_CURSOR
