@@ -1,12 +1,12 @@
 /****************************************************************************
-** $Id:  qt/qwindowdefs.h   3.0.3   edited Nov 22 02:46 $
+** $Id: qt/qwindowdefs.h   3.3.5   edited Aug 31 12:17 $
 **
 ** Definition of general window system dependent functions, types and
 ** constants
 **
 ** Created : 931029
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -45,8 +45,6 @@
 #include "qnamespace.h"
 #endif // QT_H
 
-#include <limits.h>
-
 // Class forward definitions
 
 class QPaintDevice;
@@ -92,22 +90,25 @@ class QWidgetListIt;
 // Window system dependent definitions
 
 #if defined(Q_WS_MAC)
+#if QT_MACOSX_VERSION < 0x1020
+typedef struct OpaqueEventLoopTimerRef* EventLoopTimerRef;
+typedef struct OpaqueMenuHandle *MenuRef;
+#else
+typedef struct __EventLoopTimer*        EventLoopTimerRef;
+typedef struct OpaqueMenuRef*        MenuRef;
+#endif
 
-#ifndef QMAC_NO_QUARTZ
+#ifndef Q_WS_MACX
 typedef struct CGContext *CGContextRef;
 #endif
-typedef struct OpaqueControlRef *ControlRef;
 typedef struct OpaqueWindowGroupRef *WindowGroupRef;
 typedef struct OpaqueGrafPtr *CGrafPtr;
-typedef struct OpaqueMenuHandle *MenuRef;
 typedef struct OpaquePMPrintSession *PMPrintSession;
 typedef struct OpaquePMPrintSettings *PMPrintSettings;
 typedef struct OpaquePMPageFormat *PMPageFormat;
 typedef struct Point Point;
-typedef struct FSSpec FSSpec;
 typedef struct OpaqueEventHandlerRef*   EventHandlerRef;
 typedef struct OpaqueEventHandlerCallRef*  EventHandlerCallRef;
-typedef struct OpaqueEventLoopTimerRef* EventLoopTimerRef;
 typedef struct OpaqueEventRef*          EventRef;
 typedef long int OSStatus;
 typedef struct OpaqueScrapRef *ScrapRef;
@@ -122,14 +123,19 @@ typedef struct BitMap BitMap;
 typedef struct EventRecord EventRecord;
 typedef void * MSG;
 typedef int WId;
+typedef struct AEDesc AppleEvent;
 
-#endif
+#endif // Q_WS_MAC
 
 #if defined(Q_WS_WIN)
 #include "qwindowdefs_win.h"
 #endif // Q_WS_WIN
 
 
+#if defined(Q_OS_TEMP)
+#include "qwinfunctions_wce.h"
+#endif // Q_OS_TEMP
+
 #if defined(Q_WS_X11)
 
 typedef struct _XDisplay Display;
@@ -140,10 +146,13 @@ typedef unsigned long  WId;
 
 Q_EXPORT Display *qt_xdisplay();
 Q_EXPORT int	 qt_xscreen();
-Q_EXPORT WId	 qt_xrootwin();
+Q_EXPORT WId	 qt_xrootwin(); // ### REMOVE 4.0
+Q_EXPORT WId	 qt_xrootwin( int scrn ); // ### 4.0 add default arg of -1
 Q_EXPORT GC	 qt_xget_readonly_gc( int scrn, bool monochrome );
 Q_EXPORT GC	 qt_xget_temp_gc( int scrn, bool monochrome );
 
+Q_EXPORT const char *qAppClass();		// get application class
+
 #endif // Q_WS_X11
 
 #if defined(Q_WS_QWS)
@@ -181,10 +190,5 @@ Q_EXPORT void qRemovePostRoutine( QtCleanUpFunction );
 typedef QtCleanUpFunction Q_CleanUpFunction;
 #endif
 
-// ### remove 3.0
-Q_EXPORT void *qt_find_obj_child( QObject *, const char *, const char * );
-#define Q_CHILD(parent,type,name) \
-	((type*)qt_find_obj_child(parent,#type,name))
-
 
 #endif // QWINDOWDEFS_H
