@@ -4,7 +4,7 @@
 **
 ** Created : 001103
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the sql module of the Qt GUI Toolkit.
 **
@@ -181,8 +181,8 @@ bool QMYSQLResult::fetch( int i )
 {
     if ( isForwardOnly() ) { // fake a forward seek
 	if ( at() < i ) {
-	    int x = i;
-	    while ( x-- && fetchNext() );
+	    int x = i - at();
+	    while ( --x && fetchNext() );
 	    return fetchNext();
 	} else {
 	    return FALSE;
@@ -237,12 +237,13 @@ QVariant QMYSQLResult::data( int field )
     QString val( d->row[field] );
     switch ( d->fieldTypes.at( field ) ) {
     case QVariant::LongLong:
-	if ( val[0] == '-' ) // signed or unsigned?
-	    return QVariant( val.toLongLong() );
-	else
-	    return QVariant( val.toULongLong() );
+	return QVariant( val.toLongLong() );
+    case QVariant::ULongLong:
+	return QVariant( val.toULongLong() );
     case QVariant::Int: 
 	return QVariant( val.toInt() );
+    case QVariant::UInt:
+	return QVariant( val.toUInt() );
     case QVariant::Double:
 	return QVariant( val.toDouble() );
     case QVariant::Date:
@@ -499,7 +500,7 @@ bool QMYSQLDriver::open( const QString& db,
 				NULL,
 				optionFlags ) )
     {
-	if ( mysql_select_db( d->mysql, db )) {
+	if ( !db.isEmpty() && mysql_select_db( d->mysql, db )) {
 	    setLastError( qMakeError("Unable open database '" + db + "'", QSqlError::Connection, d ) );
 	    mysql_close( d->mysql );
 	    setOpenError( TRUE );
@@ -737,7 +738,7 @@ QString QMYSQLDriver::formatValue( const QSqlField* field, bool trimStrings ) co
 	    // buffer has to be at least length*2+1 bytes
 	    char* buffer = new char[ ba.size() * 2 + 1 ];
 	    /*uint escapedSize =*/ mysql_escape_string( buffer, ba.data(), ba.size() );
-	    r = QString( "'%1'" ).arg( buffer );
+	    r.append("'").append(buffer).append("'");
 	    delete[] buffer;
 	}
 	break;
