@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qglobal.cpp   3.2.3   edited Oct 27 20:12 $
+** $Id: qt/qglobal.cpp   3.3.5   edited Aug 31 12:13 $
 **
 ** Global functions
 **
 ** Created : 920604
 **
-** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the tools module of the Qt GUI Toolkit.
 **
@@ -44,7 +44,7 @@
 #include <stdarg.h>
 #include <stdlib.h>
 
-#if defined(Q_CC_MSVC) && !defined(Q_OS_TEMP)
+#if defined(Q_CC_MSVC) && !defined(Q_CC_MSVC_NET) && !defined(Q_OS_TEMP)
 #include <crtdbg.h>
 #endif
 
@@ -165,6 +165,54 @@ bool qSysInfo( int *wordSize, bool *bigEndian )
 
 #include "qt_mac.h"
 
+// This function has descended from Apple Source Code (FSpLocationFromFullPath),
+// but changes have been made. [Creates a minimal alias from the full pathname]
+OSErr qt_mac_create_fsspec(const QString &file, FSSpec *spec)
+{
+    FSRef fref;
+    QCString utfs = file.utf8();
+    OSErr ret = FSPathMakeRef((const UInt8 *)utfs.data(), &fref, NULL);
+    if(ret == noErr)
+	ret = FSGetCatalogInfo(&fref, kFSCatInfoNone, NULL, NULL, spec, NULL);
+    return ret;
+}
+
+CFStringRef qstring2cfstring(const QString &str)
+{
+    return CFStringCreateWithCharacters(0, (UniChar *)str.unicode(), str.length());
+}
+
+QString cfstring2qstring(CFStringRef str)
+{
+    if(!str)
+	return QString();
+
+    CFIndex length = CFStringGetLength(str);
+    if(const UniChar *chars = CFStringGetCharactersPtr(str))
+	return QString((QChar *)chars, length);
+    UniChar *buffer = (UniChar*)malloc(length * sizeof(UniChar));
+    CFStringGetCharacters(str, CFRangeMake(0, length), buffer);
+    QString ret((QChar *)buffer, length);
+    free(buffer);
+    return ret;
+}
+
+unsigned char *p_str(const QString &s)
+{
+    CFStringRef cfstr = qstring2cfstring(s);
+    uchar *p = (uchar*)malloc(256);
+    CFStringGetPascalString(cfstr, p, 256, CFStringGetSystemEncoding());
+    CFRelease(cfstr);
+    return p;
+}
+
+QString p2qstring(const unsigned char *c) {
+    CFStringRef cfstr = CFStringCreateWithPascalString(0, c, CFStringGetSystemEncoding());
+    QString str = cfstring2qstring(cfstr);
+    CFRelease(cfstr);
+    return str;
+}
+
 int qMacVersion()
 {
     static int macver = Qt::MV_Unknown;
@@ -173,7 +221,9 @@ int qMacVersion()
 	first = FALSE;
 	long gestalt_version;
 	if(Gestalt(gestaltSystemVersion, &gestalt_version) == noErr) {
-	    if(gestalt_version >= 0x1030 && gestalt_version < 0x1040)
+            if(gestalt_version >= 0x1040 && gestalt_version < 0x1050)
+		macver = Qt::MV_10_DOT_4;
+	    else if(gestalt_version >= 0x1030 && gestalt_version < 0x1040)
 		macver = Qt::MV_10_DOT_3;
 	    else if(gestalt_version >= 0x1020 && gestalt_version < 0x1030)
 		macver = Qt::MV_10_DOT_2;
@@ -187,8 +237,30 @@ int qMacVersion()
     return macver;
 }
 Qt::MacintoshVersion qt_macver = (Qt::MacintoshVersion)qMacVersion();
-#elif defined(Q_OS_WIN32) || defined(Q_OS_CYGWIN)
+
+// HFS+ filesystems use decomposing unicode for certain layers in unicode
+// In general these don't look great as a user visible string.
+// Therefore it is a good idea to normalize them ourselves.
+// These technotes on Apple's website:
+// http://developer.apple.com/qa/qa2001/qa1235.html
+// http://developer.apple.com/qa/qa2001/qa1173.html
+QString qt_mac_precomposeFileName(const QString &str)
+{
+    if (str.isEmpty())
+        return str;
+    int strLength = str.length();
+    CFMutableStringRef cfmstr = CFStringCreateMutable(0, strLength);
+    CFStringAppendCharacters(cfmstr, (UniChar *)str.unicode(), strLength);
+    CFStringNormalize(cfmstr, kCFStringNormalizationFormC);
+    QString newStr = cfstring2qstring(cfmstr);
+    CFRelease(cfmstr);
+    return newStr;
+}
+#elif defined(Q_OS_WIN32) || defined(Q_OS_CYGWIN) || defined(Q_OS_TEMP)
 bool qt_winunicode;
+# ifdef Q_OS_TEMP
+  DWORD qt_cever = 0;
+# endif // Q_OS_TEMP
 
 #include "qt_windows.h"
 
@@ -202,20 +274,25 @@ int qWinVersion()
 #endif
 #ifndef VER_PLATFORM_WIN32_NT
 #define VER_PLATFORM_WIN32_NT	    2
+#endif
+#ifndef VER_PLATFORM_WIN32_CE
+#define VER_PLATFORM_WIN32_CE	    3
 #endif
 
     static int winver = Qt::WV_NT;
     static int t=0;
     if ( !t ) {
 	t=1;
-#ifdef Q_OS_TEMP
-	OSVERSIONINFOW osver;
-	osver.dwOSVersionInfoSize = sizeof(osver);
-	GetVersionEx( &osver );
-#else
+#ifndef Q_OS_TEMP
 	OSVERSIONINFOA osver;
 	osver.dwOSVersionInfoSize = sizeof(osver);
 	GetVersionExA( &osver );
+#else
+	OSVERSIONINFOW osver;
+	osver.dwOSVersionInfoSize = sizeof(osver);
+	GetVersionEx( &osver );
+	qt_cever = osver.dwMajorVersion * 100;
+	qt_cever += osver.dwMinorVersion * 10;
 #endif
 	switch ( osver.dwPlatformId ) {
 	case VER_PLATFORM_WIN32s:
@@ -223,18 +300,33 @@ int qWinVersion()
 	    break;
 	case VER_PLATFORM_WIN32_WINDOWS:
 	    // We treat Windows Me (minor 90) the same as Windows 98
-	    if ( ( osver.dwMinorVersion == 10 ) || ( osver.dwMinorVersion == 90 ) )
+	    if ( osver.dwMinorVersion == 90 )
+		winver = Qt::WV_Me;
+	    else if ( osver.dwMinorVersion == 10 )
 		winver = Qt::WV_98;
 	    else
 		winver = Qt::WV_95;
 	    break;
+	case VER_PLATFORM_WIN32_CE:
+#ifdef Q_OS_TEMP
+	    if ( qt_cever >= 400 )
+		winver = Qt::WV_CENET;
+	    else
+#endif
+		winver = Qt::WV_CE;
+	    break;
 	default: // VER_PLATFORM_WIN32_NT
 	    if ( osver.dwMajorVersion < 5 ) {
 		winver = Qt::WV_NT;
 	    } else if ( osver.dwMinorVersion == 0 ) {
 		winver = Qt::WV_2000;
-	    } else {
+	    } else if ( osver.dwMinorVersion == 1 ) {
 		winver = Qt::WV_XP;
+	    } else if ( osver.dwMinorVersion == 2 ) {
+		winver = Qt::WV_2003;
+	    } else {
+		qWarning("Untested Windows version detected!");
+		winver = Qt::WV_NT_based;
 	    }
 	}
     }
@@ -358,59 +450,6 @@ static QtMsgHandler handler = 0;		// pointer to debug handler
 static const int QT_BUFFER_LENGTH = 8196;	// internal buffer length
 
 
-#ifdef Q_OS_MAC
-CFStringRef qstring2cfstring(const QString &str)
-{
-    return CFStringCreateWithCharacters(0, (UniChar *)str.unicode(), str.length());
-}
-
-QString cfstring2qstring(CFStringRef str)
-{
-    if(!str)
-	return QString();
-
-    CFIndex length = CFStringGetLength(str);
-    if(const UniChar *chars = CFStringGetCharactersPtr(str))
-	return QString((QChar *)chars, length);
-    UniChar *buffer = (UniChar*)malloc(length * sizeof(UniChar));
-    CFStringGetCharacters(str, CFRangeMake(0, length), buffer);
-    QString ret((QChar *)buffer, length);
-    free(buffer);
-    return ret;
-}
-
-unsigned char * p_str(const char * c, int len=-1)
-{
-    const int maxlen = 255;
-    if(len == -1)
-	len = qstrlen(c);
-    if(len > maxlen) {
-	qWarning( "p_str len must never exceed %d", maxlen );
-	len = maxlen;
-    }
-    unsigned char *ret = (unsigned char*)malloc(len+2);
-    *ret=len;
-    memcpy(((char *)ret)+1,c,len);
-    *(ret+len+1) = '\0';
-    return ret;
-}
-
-unsigned char * p_str(const QString &s)
-{
-    return p_str(s, s.length());
-}
-
-QCString p2qstring(const unsigned char *c) {
-       char *arr = (char *)malloc(c[0] + 1);
-       memcpy(arr, c+1, c[0]);
-       arr[c[0]] = '\0';
-       QCString ret = arr;
-       delete arr;
-       return ret;
-}
-#endif
-
-
 #ifdef Q_CC_MWERKS
 
 #include "qt_mac.h"
@@ -435,23 +474,22 @@ void qDebug( const char *msg, ... )
     char buf[QT_BUFFER_LENGTH];
     va_list ap;
     va_start( ap, msg );			// use variable arg list
-    if ( handler ) {
 #if defined(QT_VSNPRINTF)
-	QT_VSNPRINTF( buf, QT_BUFFER_LENGTH, msg, ap );
+    QT_VSNPRINTF( buf, QT_BUFFER_LENGTH, msg, ap );
 #else
-	vsprintf( buf, msg, ap );
+    vsprintf( buf, msg, ap );
 #endif
-	va_end( ap );
+    va_end( ap );
+    if ( handler ) {
 	(*handler)( QtDebugMsg, buf );
     } else {
 #if defined(Q_CC_MWERKS)
-	vsprintf( buf, msg, ap );		// ### is there no vsnprintf()?
-	va_end( ap );
         mac_default_handler(buf);
+#elif defined(Q_OS_TEMP)
+	QString fstr( buf );
+	OutputDebugString( (fstr + "\n").ucs2() );
 #else
-	vfprintf( stderr, msg, ap );
-	va_end( ap );
-	fprintf( stderr, "\n" );		// add newline
+	fprintf( stderr, "%s\n", buf );		// add newline
 #endif
     }
 }
@@ -462,23 +500,22 @@ void debug( const char *msg, ... )
     char buf[QT_BUFFER_LENGTH];
     va_list ap;
     va_start( ap, msg );			// use variable arg list
-    if ( handler ) {
 #if defined(QT_VSNPRINTF)
-	QT_VSNPRINTF( buf, QT_BUFFER_LENGTH, msg, ap );
+    QT_VSNPRINTF( buf, QT_BUFFER_LENGTH, msg, ap );
 #else
-	vsprintf( buf, msg, ap );
+    vsprintf( buf, msg, ap );
 #endif
-	va_end( ap );
+    va_end( ap );
+    if ( handler ) {
 	(*handler)( QtDebugMsg, buf );
     } else {
-#ifdef Q_CC_MWERKS
-	vsprintf( buf, msg, ap );		// ### is there no vsnprintf()?
-	va_end( ap );
+#if defined(Q_CC_MWERKS)
         mac_default_handler(buf);
+#elif defined(Q_OS_TEMP)
+	QString fstr( buf );
+	OutputDebugString( (fstr + "\n").ucs2() );
 #else
-	vfprintf( stderr, msg, ap );
-	va_end( ap );
-	fprintf( stderr, "\n" );		// add newline
+	fprintf( stderr, "%s\n", buf );		// add newline
 #endif
     }
 }
@@ -488,23 +525,22 @@ void qWarning( const char *msg, ... )
     char buf[QT_BUFFER_LENGTH];
     va_list ap;
     va_start( ap, msg );			// use variable arg list
-    if ( handler ) {
 #if defined(QT_VSNPRINTF)
-	QT_VSNPRINTF( buf, QT_BUFFER_LENGTH, msg, ap );
+    QT_VSNPRINTF( buf, QT_BUFFER_LENGTH, msg, ap );
 #else
-	vsprintf( buf, msg, ap );
+    vsprintf( buf, msg, ap );
 #endif
-	va_end( ap );
+    va_end( ap );
+    if ( handler ) {
 	(*handler)( QtWarningMsg, buf );
     } else {
-#ifdef Q_CC_MWERKS
-	vsprintf( buf, msg, ap );		// ### is there no vsnprintf()?
-	va_end( ap );
+#if defined(Q_CC_MWERKS)
         mac_default_handler(buf);
+#elif defined(Q_OS_TEMP)
+	QString fstr( buf );
+	OutputDebugString( (fstr + "\n").ucs2() );
 #else
-	vfprintf( stderr, msg, ap );
-	va_end( ap );
-	fprintf( stderr, "\n" );		// add newline
+	fprintf( stderr, "%s\n", buf );		// add newline
 #endif
     }
 }
@@ -516,23 +552,22 @@ void warning( const char *msg, ... )
     char buf[QT_BUFFER_LENGTH];
     va_list ap;
     va_start( ap, msg );			// use variable arg list
-    if ( handler ) {
 #if defined(QT_VSNPRINTF)
-	QT_VSNPRINTF( buf, QT_BUFFER_LENGTH, msg, ap );
+    QT_VSNPRINTF( buf, QT_BUFFER_LENGTH, msg, ap );
 #else
-	vsprintf( buf, msg, ap );
+    vsprintf( buf, msg, ap );
 #endif
-	va_end( ap );
+    va_end( ap );
+    if ( handler ) {
 	(*handler)( QtWarningMsg, buf );
     } else {
-#ifdef Q_CC_MWERKS
-	vsprintf( buf, msg, ap );		// ### is there no vsnprintf()?
-	va_end( ap );
+#if defined(Q_CC_MWERKS)
         mac_default_handler(buf);
+#elif defined(Q_OS_TEMP)
+	QString fstr( buf );
+	OutputDebugString( (fstr + "\n").ucs2() );
 #else
-	vfprintf( stderr, msg, ap );
-	va_end( ap );
-	fprintf( stderr, "\n" );		// add newline
+	fprintf( stderr, "%s\n", buf );		// add newline
 #endif
     }
 }
@@ -542,31 +577,27 @@ void qFatal( const char *msg, ... )
     char buf[QT_BUFFER_LENGTH];
     va_list ap;
     va_start( ap, msg );			// use variable arg list
-    if ( handler ) {
 #if defined(QT_VSNPRINTF)
-	QT_VSNPRINTF( buf, QT_BUFFER_LENGTH, msg, ap );
+    QT_VSNPRINTF( buf, QT_BUFFER_LENGTH, msg, ap );
 #else
-	vsprintf( buf, msg, ap );
+    vsprintf( buf, msg, ap );
 #endif
-	va_end( ap );
+    va_end( ap );
+    if ( handler ) {
 	(*handler)( QtFatalMsg, buf );
     } else {
-#ifdef Q_CC_MWERKS
-	vsprintf( buf, msg, ap );		// ### is there no vsnprintf()?
-	va_end( ap );
+#if defined(Q_CC_MWERKS)
         mac_default_handler(buf);
 #else
-	vsprintf( buf, msg, ap );
-	va_end( ap );
 	fprintf( stderr, "%s\n", buf );		// add newline
 #endif
 #if defined(Q_OS_UNIX) && defined(QT_DEBUG)
 	abort();				// trap; generates core dump
-#elif defined(Q_OS_TEMP) && defined(_DEBUG)
+#elif defined(Q_OS_TEMP) && defined(QT_DEBUG)
 	QString fstr;
-	fstr.sprintf( "%s:%s %s %s", __FILE__, __LINE__, QT_VERSION_STR, buf );
+	fstr.sprintf( "%s:%s %s %s\n", __FILE__, __LINE__, QT_VERSION_STR, buf );
 	OutputDebugString( fstr.ucs2() );
-#elif defined(Q_CC_MSVC) && defined(_DEBUG)
+#elif defined(_CRT_ERROR) && defined(_DEBUG)
 	_CrtDbgReport( _CRT_ERROR, __FILE__, __LINE__, QT_VERSION_STR, buf );
 #else
 	exit( 1 );				// goodbye cruel world
@@ -580,31 +611,27 @@ void fatal( const char *msg, ... )
     char buf[QT_BUFFER_LENGTH];
     va_list ap;
     va_start( ap, msg );			// use variable arg list
-    if ( handler ) {
 #if defined(QT_VSNPRINTF)
-	QT_VSNPRINTF( buf, QT_BUFFER_LENGTH, msg, ap );
+    QT_VSNPRINTF( buf, QT_BUFFER_LENGTH, msg, ap );
 #else
-	vsprintf( buf, msg, ap );
+    vsprintf( buf, msg, ap );
 #endif
-	va_end( ap );
+    va_end( ap );
+    if ( handler ) {
 	(*handler)( QtFatalMsg, buf );
     } else {
-#ifdef Q_CC_MWERKS
-	vsprintf( buf, msg, ap );		// ### is there no vsnprintf()?
-	va_end( ap );
+#if defined(Q_CC_MWERKS)
         mac_default_handler(buf);
 #else
-	vfprintf( stderr, msg, ap );
-	va_end( ap );
-	fprintf( stderr, "\n" );		// add newline
+	fprintf( stderr, "%s\n", buf );		// add newline
 #endif
 #if defined(Q_OS_UNIX) && defined(QT_DEBUG)
 	abort();				// trap; generates core dump
-#elif defined(Q_OS_TEMP) && defined(_DEBUG)
+#elif defined(Q_OS_TEMP) && defined(QT_DEBUG)
 	QString fstr;
-	fstr.sprintf( "%s:%s %s %s", __FILE__, __LINE__, QT_VERSION_STR, buf );
+	fstr.sprintf( "%s:%s %s %s\n", __FILE__, __LINE__, QT_VERSION_STR, buf );
 	OutputDebugString( fstr.ucs2() );
-#elif defined(Q_CC_MSVC) && defined(_DEBUG)
+#elif defined(_CRT_ERROR) && defined(_DEBUG)
 	_CrtDbgReport( _CRT_ERROR, __FILE__, __LINE__, QT_VERSION_STR, buf );
 #else
 	exit( 1 );				// goodbye cruel world
