@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qucom.cpp   3.2.3   edited May 13 09:08 $
+** $Id: qt/qucom.cpp   3.3.5   edited Aug 31 12:13 $
 **
 ** Implementation of the QUcom classes
 **
 ** Created : 990101
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the tools module of the Qt GUI Toolkit.
 **
@@ -47,9 +47,9 @@ bool QUType_Null::canConvertFrom( QUObject *, QUType * ) { return FALSE; }
 bool QUType_Null::canConvertTo( QUObject *, QUType * ) { return FALSE; }
 bool QUType_Null::convertFrom( QUObject *, QUType * ) { return FALSE; }
 bool QUType_Null::convertTo( QUObject *, QUType * ) { return FALSE; }
-void QUType_Null::clear( QUObject *) {};
+void QUType_Null::clear( QUObject *) {}
 int QUType_Null::serializeTo( QUObject *, QUBuffer * ) { return 0; }
-int QUType_Null::serializeFrom( QUObject *, QUBuffer * ) { return 0; };
+int QUType_Null::serializeFrom( QUObject *, QUBuffer * ) { return 0; }
 QUType_Null static_QUType_Null;
 
 
