@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qscrollview.cpp   3.2.3   edited Aug 21 14:17 $
+** $Id: qt/qscrollview.cpp   3.3.5   edited Aug 31 12:13 $
 **
 ** Implementation of QScrollView class
 **
 ** Created : 950524
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -263,9 +263,7 @@ void QScrollViewData::hideOrShowAll(QScrollView* sv, bool isScroll )
 	int nx = ( viewport->width() - clipped_viewport->width() ) / 2;
 	int ny = ( viewport->height() - clipped_viewport->height() ) / 2;
 	clipped_viewport->move(nx,ny);
-	// no need to update, we'll receive a paintevent after move
-	// (with the safe assumption that the newly exposed area
-	// covers the entire viewport)
+        clipped_viewport->update();
     }
     for (QSVChildRec *r = children.first(); r; r=children.next()) {
 	r->hideOrShow(sv, clipped_viewport);
@@ -480,7 +478,7 @@ void QScrollViewData::viewportResized( int w, int h )
     as you would subclass other widgets, but rather than
     reimplementing mousePressEvent(), reimplement
     contentsMousePressEvent() instead. The contents specific event
-    handers provide translated events in the coordinate system of the
+    handlers provide translated events in the coordinate system of the
     scrollview. If you reimplement mousePressEvent(), you'll get
     called only when part of the QScrollView is clicked: and the only
     such part is the "corner" (if you don't set a cornerWidget()) and
@@ -578,7 +576,7 @@ void QScrollViewData::viewportResized( int w, int h )
 */
 
 QScrollView::QScrollView( QWidget *parent, const char *name, WFlags f ) :
-    QFrame( parent, name, f & (~WStaticContents) & (~WRepaintNoErase) & (~WResizeNoErase) )
+    QFrame( parent, name, f & (~WStaticContents) & (~WResizeNoErase) )
 {
     WFlags flags = WResizeNoErase | (f&WPaintClever) | (f&WRepaintNoErase) | (f&WStaticContents);
     d = new QScrollViewData( this, flags );
@@ -588,10 +586,10 @@ QScrollView::QScrollView( QWidget *parent, const char *name, WFlags f ) :
              this, SLOT( doDragAutoScroll() ) );
 #endif
 
-    connect( d->hbar, SIGNAL( valueChanged( int ) ),
-        this, SLOT( hslide( int ) ) );
-    connect( d->vbar, SIGNAL( valueChanged( int ) ),
-        this, SLOT( vslide( int ) ) );
+    connect( d->hbar, SIGNAL( valueChanged(int) ),
+        this, SLOT( hslide(int) ) );
+    connect( d->vbar, SIGNAL( valueChanged(int) ),
+        this, SLOT( vslide(int) ) );
 
     connect( d->hbar, SIGNAL(sliderPressed()), this, SLOT(hbarIsPressed()) );
     connect( d->hbar, SIGNAL(sliderReleased()), this, SLOT(hbarIsReleased()) );
@@ -1180,12 +1178,12 @@ void  QScrollView::mouseMoveEvent( QMouseEvent *e ) //#### remove for 4.0
 void QScrollView::wheelEvent( QWheelEvent *e )
 {
     QWheelEvent ce( viewport()->mapFromGlobal( e->globalPos() ),
-                    e->globalPos(), e->delta(), e->state());
+                    e->globalPos(), e->delta(), e->state(), e->orientation());
     viewportWheelEvent(&ce);
     if ( !ce.isAccepted() ) {
-	if ( e->orientation() == Horizontal && horizontalScrollBar() )
+	if ( e->orientation() == Horizontal && horizontalScrollBar() && horizontalScrollBar()->isEnabled() )
 	    QApplication::sendEvent( horizontalScrollBar(), e);
-	else  if (e->orientation() == Vertical && verticalScrollBar() )
+	else  if (e->orientation() == Vertical && verticalScrollBar() && verticalScrollBar()->isEnabled() )
 	    QApplication::sendEvent( verticalScrollBar(), e);
     } else {
 	e->accept();
@@ -1587,10 +1585,12 @@ void QScrollView::contentsMouseReleaseEvent( QMouseEvent* e )
     This event handler is called whenever the QScrollView receives a
     mouseDoubleClickEvent(): the click position in \a e is translated to be a
     point on the contents.
+
+    The default implementation generates a normal mouse press event.
 */
 void QScrollView::contentsMouseDoubleClickEvent( QMouseEvent* e )
 {
-    e->ignore();
+    contentsMousePressEvent(e);		    // try mouse press event
 }
 
 /*!
@@ -2447,7 +2447,7 @@ void QScrollView::changeFrameRect(const QRect& r)
     right and \a bottom. This is useful for applications such as
     spreadsheets with "locked" rows and columns. The marginal space is
     \e inside the frameRect() and is left blank; reimplement
-    drawContents() or put widgets in the unused area.
+    drawFrame() or put widgets in the unused area.
 
     By default all margins are zero.
 
@@ -2698,9 +2698,9 @@ QSize QScrollView::sizeHint() const
     } else {
 	sz += QSize( d->contentsWidth(), contentsHeight() );
     }
-    if (d->hMode == AlwaysOn)
-	sz.setWidth(sz.width() + d->hbar->sizeHint().width());
     if (d->vMode == AlwaysOn)
+	sz.setWidth(sz.width() + d->vbar->sizeHint().width());
+    if (d->hMode == AlwaysOn)
 	sz.setHeight(sz.height() + d->hbar->sizeHint().height());
     return sz.expandedTo( QSize(12 * h, 8 * h) )
 	     .boundedTo( QSize(36 * h, 24 * h) );
@@ -2795,6 +2795,12 @@ void QScrollView::doDragAutoScroll()
     moves the cursor close to a border of the view. Of course this
     works only if the viewport accepts drops. Specifying FALSE
     disables this autoscroll feature.
+
+    \warning Enabling this property might not be enough to
+    effectively turn on autoscrolling. If you put a custom widget in
+    the QScrollView, you might need to call QDragEvent::ignore() on
+    the event in the dragEnterEvent() and dragMoveEvent()
+    reimplementations.
 */
 
 void QScrollView::setDragAutoScroll( bool b )
