@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qslider.cpp   3.2.3   edited May 13 09:08 $
+** $Id: qt/qslider.cpp   3.3.5   edited Aug 31 12:13 $
 **
 ** Implementation of QSlider class
 **
 ** Created : 961019
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -446,7 +446,7 @@ void QSlider::mousePressEvent( QMouseEvent *e )
 	clickOffset = slideLength / 2;
     } else if ( orient == Horizontal && e->pos().x() < r.left() //### goodPart
 		|| orient == Vertical && e->pos().y() < r.top() ) {
-	if ( QApplication::reverseLayout() ) {
+	if ( orient == Horizontal && QApplication::reverseLayout() ) {
 	    state = TimingUp;
 	    addPage();
 	} else {
@@ -459,7 +459,7 @@ void QSlider::mousePressEvent( QMouseEvent *e )
 	timer->start( thresholdTime, TRUE );
     } else if ( orient == Horizontal && e->pos().x() > r.right() //### goodPart
 		|| orient == Vertical && e->pos().y() > r.bottom() ) {
-	if ( QApplication::reverseLayout() ) {
+	if ( orient == Horizontal && QApplication::reverseLayout() ) {
 	    state = TimingDown;
 	    subtractPage();
 	} else {
@@ -612,12 +612,20 @@ void QSlider::keyPressEvent( QKeyEvent *e )
     bool sloppy = bool(style().styleHint(QStyle::SH_Slider_SloppyKeyEvents, this));
     switch ( e->key() ) {
     case Key_Left:
-	if ( sloppy || orient == Horizontal )
-	    subtractLine();
+	if ( sloppy || orient == Horizontal ) {
+	    if (QApplication::reverseLayout())
+		addLine();
+	    else
+		subtractLine();
+	}
 	break;
     case Key_Right:
-	if ( sloppy || orient == Horizontal )
-	    addLine();
+	if ( sloppy || orient == Horizontal ) {
+	    if (QApplication::reverseLayout())
+		subtractLine();
+	    else
+		addLine();
+	}
 	break;
     case Key_Up:
 	if ( sloppy || orient == Vertical )
