@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qwidgetstack.cpp   3.2.3   edited Jul 25 15:22 $
+** $Id: qt/qwidgetstack.cpp   3.3.5   edited Aug 31 12:13 $
 **
 ** Implementation of QWidgetStack class
 **
 ** Created : 980128
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -185,9 +185,9 @@ QWidgetStack::~QWidgetStack()
     If you pass an id \>= 0 this ID is used. If you pass an \a id of
     -1 (the default), the widgets will be numbered automatically. If
     you pass -2 a unique negative integer will be generated. No widget
-    has an ID of -1. Returns the ID or -1 on failure (e.g. \w is 0).
+    has an ID of -1. Returns the ID or -1 on failure (e.g. \a w is 0).
 
-    If you pass an id that is already used, then a unique negative 
+    If you pass an id that is already used, then a unique negative
     integer will be generated to prevent two widgets having the same
     id.
 
@@ -312,6 +312,12 @@ void QWidgetStack::raiseWidget( QWidget *w )
 	return;
     }
 
+    if (w->maximumSize().width() < invisible->width()
+        || w->maximumSize().height() < invisible->height())
+        invisible->setBackgroundMode(backgroundMode());
+    else if (invisible->backgroundMode() != NoBackground)
+        invisible->setBackgroundMode(NoBackground);
+
     if ( invisible->isHidden() ) {
 	invisible->setGeometry( contentsRect() );
 	invisible->lower();
