@@ -1,5 +1,5 @@
 /**********************************************************************
-** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 2000-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the Qt Assistant.
 **
@@ -34,7 +34,6 @@
 #include <qpixmap.h>
 #include <qptrlist.h>
 #include <qstringlist.h>
-#include <qsettings.h>
 #include <qdir.h>
 #include <qmessagebox.h>
 #include <qguardedptr.h>
@@ -140,14 +139,15 @@ int main( int argc, char ** argv )
     if ( argc > 1 ) {
 	QString arg( argv[1] );
 	arg = arg.lower();
+	if ( arg == "-addcontentfile"
+	    || arg == "-removecontentfile"
 #ifndef Q_WS_WIN
-	if ( arg == "-addcontentfile" ||
-	     arg == "-removecontentfile" ||
-	     arg == "-help" )
-	    withGUI = FALSE;
+	    || arg == "-help"
 #endif
+	    )
+	    withGUI = FALSE;
     }
-    QApplication a( argc, argv, withGUI );
+    QApplication a(argc, argv, withGUI);
 
     QString resourceDir;
     AssistantServer *as = 0;
@@ -198,19 +198,46 @@ int main( int argc, char ** argv )
 	    } else if ( QString( argv[i] ).lower() == "-removecontentfile" ) {
 		INDEX_CHECK( "Missing content file!" );
 		Config *c = Config::loadConfig( QString::null );
-		QFileInfo file( argv[i+1] );
-		if( !file.exists() ) {
-		    fprintf( stderr, "Could not locate content file: '%s'\n",
-			     file.absFilePath().latin1() );
-		    fflush( stderr );
-		    return 1;
-		}
 		Profile *profile = c->profile();
-		profile->removeDocFileEntry( file.absFilePath() );
-		c->setDocRebuild( TRUE );
-		c->save();
+		QStringList entries = profile->docs.grep(argv[i+1]);
+		if (entries.count() == 0) {
+		    fprintf(stderr, "Could not locate content file: '%s'\n",
+			    argv[i+1]);
+		    fflush(stderr);
+		    return 1;
+        } else if (entries.count() > 1) {
+		    fprintf(stderr, "More than one entry matching file name found, "
+			"please specify full path to file");
+		    fflush(stderr);
+		    return 1;
+        } else {
+		    QFileInfo file(entries[0]);
+		    if( !file.exists() ) {
+			fprintf( stderr, "Could not locate content file: '%s'\n",
+			    file.absFilePath().latin1() );
+			fflush( stderr );
+			return 1;
+		    }
+		    profile->removeDocFileEntry( file.absFilePath() );
+		    c->setDocRebuild( TRUE );
+		    c->save();
+        }
 		return 0;
-	    } else if ( QString( argv[i] ).lower() == "-hidesidebar" ) {
+        } else if ( QString( argv[i] ).lower() == "-docpath" ) {
+            INDEX_CHECK( "Missing path!" );
+            QDir dir( argv[i+1] );
+            if ( dir.exists() ) {
+                Config *c = Config::loadConfig( QString::null );
+                c->saveProfile(Profile::createDefaultProfile(dir.absPath()));
+                c->loadDefaultProfile();
+                c->setDocRebuild( TRUE );
+                c->save();
+            } else {
+                fprintf( stderr, "The specified path does not exist!\n");
+                fflush( stderr );
+                return 1;
+            }
+        } else if ( QString( argv[i] ).lower() == "-hidesidebar" ) {
 		hideSidebar = TRUE;
 	    } else if ( QString( argv[i] ).lower() == "-help" ) {
 		QString helpText( "Usage: assistant [option]\n"
@@ -224,7 +251,11 @@ int main( int argc, char ** argv )
 				  "                            documentation available by default\n"
 				  " -removeContentFile file    removes the content file 'file' from the\n"
 				  "                            documentation available by default\n"
+                  " -docPath path              sets the Qt documentation root path to\n"
+                  "                            'path' and starts assistant\n"
 				  " -hideSidebar               assistant will hide the sidebar.\n"
+				  " -resourceDir               assistant will load translations from\n"
+                  "                            this directory.\n"
 				  " -help                      shows this help.");
 #ifdef Q_WS_WIN
 		QMessageBox::information( 0, "Qt Assistant", "<pre>" + helpText + "</pre>" );
@@ -245,8 +276,6 @@ int main( int argc, char ** argv )
 
     if( resourceDir.isNull() )
 	resourceDir = qInstallPathTranslations();
-    if( !QFile::exists( resourceDir ) )
-	fprintf( stderr, "Resource file directory '%s' does not exist!\n", resourceDir.latin1() );
 
     QTranslator translator( 0 );
     translator.load( QString("assistant_") + QTextCodec::locale(), resourceDir );
@@ -264,7 +293,7 @@ int main( int argc, char ** argv )
     }
 
     bool max = conf->isMaximized();
-    QString link = conf->source();
+    QStringList links = conf->source();
     conf->hideSideBar( hideSidebar );
 
     QGuardedPtr<MainWindow> mw = new MainWindow( 0, "Assistant" );
@@ -285,7 +314,7 @@ int main( int argc, char ** argv )
     if ( !file.isEmpty() )
 	mw->showLink( file );
     else if ( file.isEmpty() )
-	mw->showLink( link );
+	mw->showLinks( links );
 
     a.connect( &a, SIGNAL( lastWindowClosed() ), &a, SLOT( quit() ) );
 
