@@ -1,5 +1,5 @@
 /**********************************************************************
-** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 2000-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of Qt Designer.
 **
@@ -362,13 +362,18 @@ DeleteCommand::DeleteCommand( const QString &n, FormWindow *fw,
     : Command( n, fw ), widgets( wl )
 {
     widgets.setAutoDelete( FALSE );
-
+    QWidgetList copyOfWidgets = widgets;
+    copyOfWidgets.setAutoDelete(FALSE);
+    
     // Include the children of the selected items when deleting
     for ( QWidget *w = widgets.first(); w; w = widgets.next() ) {
 	QObjectList *children = w->queryList( "QWidget" );
 	for ( QWidget *c = (QWidget *)children->first(); c; c = (QWidget *)children->next() ) {
-	    if ( widgets.find( c ) == -1 && formWindow()->widgets()->find( c ) )
-		widgets.append( c );
+	    if ( copyOfWidgets.find( c ) == -1 && formWindow()->widgets()->find( c ) ) {
+		widgets.insert(widgets.at() + 1, c);
+		widgets.prev();
+		copyOfWidgets.append(c);
+	    }
 	}
 	delete children;
     }
@@ -544,6 +549,9 @@ bool SetPropertyCommand::checkProperty()
 
 void SetPropertyCommand::setProperty( const QVariant &v, const QString &currentItemText, bool select )
 {
+    if ( !widget )
+        return;
+
     if ( !formWindow()->isWidgetSelected( widget ) && !formWindow()->isMainContainer( widget ) && select )
 	formWindow()->selectWidget( widget );
     if ( editor->widget() != widget && select )
@@ -1796,8 +1804,8 @@ void AddActionToToolBarCommand::execute()
 	toolBar->reInsert();
 	QObject::connect( action, SIGNAL( destroyed() ), toolBar, SLOT( actionRemoved() ) );
     } else {
-	QObjectListIt it( *action->children() );
 	if ( action->children() ) {
+	    QObjectListIt it( *action->children() );
 	    int i = 0;
 	    while ( it.current() ) {
 		QObject *o = it.current();
@@ -2023,9 +2031,7 @@ void RenameContainerPageCommand::unexecute()
     formWindow()->emitUpdateProperties( formWindow()->currentWidget() );
     // #### show and update pages in object hierarchy view
 }
-
-#endif
-
+#endif // QT_CONTAINER_CUSTOM_WIDGETS
 // ------------------------------------------------------------
 
 AddActionToPopupCommand::AddActionToPopupCommand( const QString &n,
@@ -2113,8 +2119,12 @@ RenameActionCommand::RenameActionCommand( const QString &n,
 
 void RenameActionCommand::execute()
 {
-    action->setMenuText( newName );
-    action->setText( newName );
+    QString actionText = newName;
+    actionText.replace("&&", "&");
+    QString menuText = newName;
+    action->setText( actionText );
+    action->setMenuText( menuText );
+
     formWindow()->mainWindow()->objectHierarchy()->rebuild();
 }
 
@@ -2181,22 +2191,20 @@ void AddMenuCommand::execute()
     QMainWindow *mw = (QMainWindow*)formWindow()->mainContainer();
     if ( !mb ) {
 	mb = new MenuBarEditor( formWindow(), mw );
-	QString n = "MenuBarEditor";
-	formWindow()->unify( mb, n, TRUE );
-	mb->setName( n );
-	MetaDataBase::addEntry( mb );
+	mb->setName( "MenuBarEditor" );
+	formWindow()->insertWidget( mb, TRUE );
     }
     if ( !item ) {
 	PopupMenuEditor *popup = new PopupMenuEditor( formWindow(), mw );
-	QString n = "PopupMenuEditor";
-	formWindow()->unify( popup, n, TRUE );
-	popup->setName( n );
-	MetaDataBase::addEntry( popup );
+	popup->setName( "PopupMenuEditor" );
+	formWindow()->insertWidget( popup, TRUE );
 	mb->insertItem( name, popup, index );
 	index = mb->findItem( popup );
 	item = mb->item( index );
     } else {
-	MetaDataBase::addEntry( item->menu() );
+	PopupMenuEditor *popup = item->menu();
+	popup->setName( item->menuText() );
+	formWindow()->insertWidget( popup, TRUE );
 	mb->insertItem( item, index );
     }
     formWindow()->mainWindow()->objectHierarchy()->rebuild();
@@ -2210,9 +2218,9 @@ void AddMenuCommand::unexecute()
     }
     item->menu()->hide();
     int i = mb->findItem( item );
+    formWindow()->removeWidget( item->menu() );
     mb->removeItemAt( i );
     formWindow()->mainWindow()->objectHierarchy()->rebuild();
-    MetaDataBase::removeEntry( item->menu() );
 }
 
 // ------------------------------------------------------------
@@ -2294,7 +2302,7 @@ RenameMenuCommand::RenameMenuCommand( const QString &n,
 				      MenuBarEditorItem *i )
     : Command( n, fw ), bar( b ), item( i ), newName( nm )
 {
-    oldName = item->menuText();
+    oldName = item->menu()->name();
 }
 
 QString RenameMenuCommand::makeLegal( const QString &str )
@@ -2306,7 +2314,8 @@ QString RenameMenuCommand::makeLegal( const QString &str )
 	c = str.at(i).latin1();
 	if ( c == '-' || c == ' ' )
 	    d += '_';
-	else if ( ( c > 47 && c < 91 ) || c == 95 || ( c > 96 && c < 123 ) )
+	else if ( ( c >= '0' && c <= '9') || ( c >= 'A' && c <= 'Z' )
+                  || c == '_' || ( c >= 'a' && c <= 'z' ) )
 	    d += c;
 	++i;
     }
@@ -2315,21 +2324,19 @@ QString RenameMenuCommand::makeLegal( const QString &str )
 
 void RenameMenuCommand::execute()
 {
-    MetaDataBase::removeEntry( item );
+    PopupMenuEditor *popup = item->menu();
     item->setMenuText( newName );
     QString legal = makeLegal( newName );
-    formWindow()->unify( item, legal, FALSE );
-    item->setName( legal );
-    MetaDataBase::addEntry( item );
+    formWindow()->unify( popup, legal, TRUE );
+    popup->setName( legal );
     formWindow()->mainWindow()->objectHierarchy()->rebuild();
 }
 
 void RenameMenuCommand::unexecute()
 {
-    MetaDataBase::removeEntry( item );
+    PopupMenuEditor *popup = item->menu();
     item->setMenuText( oldName );
-    item->setName( oldName );
-    MetaDataBase::addEntry( item );
+    popup->setName( oldName );
     formWindow()->mainWindow()->objectHierarchy()->rebuild();
 }
 
