@@ -1,5 +1,5 @@
 /**********************************************************************
-** Copyright (C) 2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of Qt Designer.
 **
@@ -8,10 +8,16 @@
 ** Foundation and appearing in the file LICENSE.GPL included in the
 ** packaging of this file.
 **
+** Licensees holding valid Qt Enterprise Edition or Qt Professional Edition
+** licenses may use this file in accordance with the Qt Commercial License
+** Agreement provided with the Software.
+**
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ** See http://www.trolltech.com/gpl/ for GPL licensing information.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+**   information about Qt Commercial License Agreements.
 **
 ** Contact info@trolltech.com if any conditions of this licensing are
 ** not clear to you.
@@ -29,6 +35,8 @@
 #include <qpen.h>
 #include <qbitmap.h>
 #include <qsplitter.h>
+#include <qvaluevector.h>
+#include <qmainwindow.h>
 
 bool operator<( const QGuardedPtr<QWidget> &p1, const QGuardedPtr<QWidget> &p2 )
 {
@@ -159,7 +167,7 @@ bool Layout::prepareLayout( bool &needMove, bool &needReparent )
     for ( QWidget *w = widgets.first(); w; w = widgets.next() )
 	w->raise();
     needMove = !layoutBase;
-    needReparent = needMove || layoutBase->inherits( "QLayoutWidget" ) || layoutBase->inherits( "QSplitter" );
+    needReparent = needMove || ::qt_cast<QLayoutWidget*>(layoutBase) || ::qt_cast<QSplitter*>(layoutBase);
     if ( !layoutBase ) {
 	if ( !useSplitter )
 	    layoutBase = WidgetFactory::create( WidgetDatabase::idFromClassName( "QLayoutWidget" ),
@@ -208,7 +216,7 @@ void Layout::undoLayout()
     }
     formWindow->selectWidget( layoutBase, FALSE );
     WidgetFactory::deleteLayout( layoutBase );
-    if ( parent != layoutBase && !layoutBase->inherits( "QMainWindow" ) ) {
+    if ( parent != layoutBase && !::qt_cast<QMainWindow*>(layoutBase) ) {
 	layoutBase->hide();
 	QString n = layoutBase->name();
 	n.prepend( "qt_dead_widget_" );
@@ -313,13 +321,13 @@ void HorizontalLayout::doLayout()
 		layout->addWidget( w, 0, ( (Spacer*)w )->alignment() );
 	    else
 		layout->addWidget( w );
-	    if ( w->inherits( "QLayoutWidget" ) )
+	    if ( ::qt_cast<QLayoutWidget*>(w) )
 		( (QLayoutWidget*)w )->updateSizePolicy();
 	}
 	w->show();
     }
 
-    if ( layoutBase->inherits( "QSplitter" ) )
+    if ( ::qt_cast<QSplitter*>(layoutBase) )
 	( (QSplitter*)layoutBase )->setOrientation( Qt::Horizontal );
 
     finishLayout( needMove, layout );
@@ -377,13 +385,13 @@ void VerticalLayout::doLayout()
 		layout->addWidget( w, 0, ( (Spacer*)w )->alignment() );
 	    else
 		layout->addWidget( w );
-	    if ( w->inherits( "QLayoutWidget" ) )
+	    if ( ::qt_cast<QLayoutWidget*>(w) )
 		( (QLayoutWidget*)w )->updateSizePolicy();
 	}
 	w->show();
     }
 
-    if ( layoutBase->inherits( "QSplitter" ) )
+    if ( ::qt_cast<QSplitter*>(layoutBase) )
 	( (QSplitter*)layoutBase )->setOrientation( Qt::Vertical );
 
     finishLayout( needMove, layout );
@@ -401,6 +409,12 @@ public:
 
     QWidget* cell( int row, int col ) const { return cells[ row * ncols + col]; }
     void setCell( int row, int col, QWidget* w ) { cells[ row*ncols + col] = w; }
+    void setCells( QRect c, QWidget* w ) {
+	for ( int rows = c.bottom()-c.top(); rows >= 0; rows--)
+	    for ( int cols = c.right()-c.left(); cols >= 0; cols--) {
+		setCell(c.top()+rows, c.left()+cols, w);
+	    }
+    }
     int numRows() const { return nrows; }
     int numCols() const { return ncols; }
 
@@ -745,11 +759,11 @@ void GridLayout::doLayout()
 	    if ( needReparent && w->parent() != layoutBase )
 		w->reparent( layoutBase, 0, QPoint( 0, 0 ), FALSE );
 	    if ( rs * cs == 1 ) {
-		layout->addWidget( w, r, c, w->inherits( "Spacer" ) ? ( (Spacer*)w )->alignment() : 0 );
+		layout->addWidget( w, r, c, ::qt_cast<Spacer*>(w) ? ( (Spacer*)w )->alignment() : 0 );
 	    } else {
-		layout->addMultiCellWidget( w, r, r+rs-1, c, c+cs-1, w->inherits( "Spacer" ) ? ( (Spacer*)w )->alignment() : 0 );
+		layout->addMultiCellWidget( w, r, r+rs-1, c, c+cs-1, ::qt_cast<Spacer*>(w) ? ( (Spacer*)w )->alignment() : 0 );
 	    }
-	    if ( w->inherits( "QLayoutWidget" ) )
+	    if ( ::qt_cast<QLayoutWidget*>(w) )
 		( (QLayoutWidget*)w )->updateSizePolicy();
 	    w->show();
 	} else {
@@ -767,30 +781,78 @@ void GridLayout::setup()
 
 void GridLayout::buildGrid()
 {
-    QWidget* w;
-    QRect br;
-    for ( w = widgets.first(); w; w = widgets.next() )
-	br = br.unite( w->geometry() );
+    if ( !widgets.count() )
+	return;
 
+    // Pixel to cell conversion:
+    // By keeping a list of start'n'stop values (x & y) for each widget,
+    // it is possible to create a very small grid of cells to represent
+    // the widget layout.
+    // -----------------------------------------------------------------
+
+    // We need a list of both start and stop values for x- & y-axis
+    QValueVector<int> x( widgets.count()*2 );
+    QValueVector<int> y( widgets.count()*2 );
+
+    // Using push_back would look nicer, but operator[] is much faster
+    int index  = 0;
+    QWidget* w = 0;
+    for ( w = widgets.first(); w; w = widgets.next() ) {
+	QRect widgetPos = w->geometry();
+	x[index]   = widgetPos.left();
+	x[index+1] = widgetPos.right();
+	y[index]   = widgetPos.top();
+	y[index+1] = widgetPos.bottom();
+	index += 2;
+    }
+
+    qHeapSort(x);
+    qHeapSort(y);
+
+    // Remove duplicate x enteries (Remove next, if equal to current)
+    if ( !x.empty() ) {
+	for (QValueVector<int>::iterator current = x.begin() ;
+	     (current != x.end()) && ((current+1) != x.end()) ; )
+	    if ( (*current == *(current+1)) )
+		x.erase(current+1);
+	    else
+		current++;
+    }
+
+    // Remove duplicate y enteries (Remove next, if equal to current)
+    if ( !y.empty() ) {
+	for (QValueVector<int>::iterator current = y.begin() ;
+	     (current != y.end()) && ((current+1) != y.end()) ; )
+	    if ( (*current == *(current+1)) )
+		y.erase(current+1);
+	    else
+		current++;
+    }
+
+    // Create the smallest grid possible to represent the current layout
+    // Since no widget will be placed in the last row and column, we'll
+    // skip them to increase speed even further
     delete grid;
-    grid = new Grid( br.height() / resolution.height() + 1,
-		     br.width() / resolution.width() + 1 );
+    grid = new Grid( y.size()-1, x.size()-1 );
 
-    int r,c;
-    for ( r = 0; r < grid->numRows(); r++ ) {
-	for ( c = 0; c < grid->numCols(); c++ ) {
-	    QPoint p( br.left() + c * resolution.width(),
-		      br.top() + r* resolution.height() );
-	    QRect cr( p, resolution );
-	    for ( w = widgets.first(); w; w = widgets.next() ) {
-		    // check that the overlap is significant
-		    QRect intersect = cr.intersect( w->geometry() );
-		    if ( intersect.size().width() > resolution.width()/2 &&
-			 intersect.size().height() > resolution.height()/2 ) {
-			grid->setCell( r, c, w );
-		    }
-	    }
+    // Mark the cells in the grid that contains a widget
+    for ( w = widgets.first(); w; w = widgets.next() ) {
+	QRect c(0,0,0,0), widgetPos = w->geometry();
+	// From left til right (not including)
+	for (uint cw=0; cw<x.size(); cw++) {
+	    if ( x[cw] == widgetPos.left() )
+		c.setLeft(cw);
+	    if ( x[cw] <  widgetPos.right())
+		c.setRight(cw);
 	}
+	// From top til bottom (not including)
+	for (uint ch=0; ch<y.size(); ch++) {
+	    if ( y[ch] == widgetPos.top()    )
+		c.setTop(ch);
+	    if ( y[ch] <  widgetPos.bottom() )
+		c.setBottom(ch);
+	}
+	grid->setCells(c, w); // Mark cellblock
     }
     grid->simplify();
 }
@@ -894,20 +956,20 @@ int Spacer::alignment() const
 
 QSize Spacer::minimumSize() const
 {
-    return QSize( 20, 20 );
-}
-
-QSize Spacer::sizeHint() const
-{
-    QSize s = sh.expandedTo( QSize(0,0) );
+    QSize s = QSize( 20,20 );
     if ( sizeType() == Expanding )
-	if ( orient == Horizontal )
+	if ( orient == Vertical )
 	    s.rheight() = 0;
 	else
 	    s.rwidth() = 0;
     return s;
 }
 
+QSize Spacer::sizeHint() const
+{
+    return sh;
+}
+
 
 void Spacer::setSizeHint( const QSize &s )
 {
