@@ -1,5 +1,5 @@
 /**********************************************************************
-** Copyright (C) 2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of Qt Designer.
 **
@@ -117,7 +117,7 @@ PopupMenuEditorItem::PopupMenuEditorItem( QAction * action, PopupMenuEditor * me
       removable( TRUE )
 {
     init();
-    if ( /*a->name() == "qt_separator_action" ||*/ a->inherits( "QSeparatorAction" ) )
+    if ( /*a->name() == "qt_separator_action" ||*/ ::qt_cast<QSeparatorAction*>(a) )
 	separator = TRUE;
     if ( a && a->children() )
  	a->installEventFilter( this );
@@ -148,7 +148,7 @@ void PopupMenuEditorItem::init()
 	QObject::connect( a, SIGNAL( destroyed() ), this, SLOT( selfDestruct() ) );
 	if ( m && !isSeparator() ) {
 	    s = new PopupMenuEditor( m->formWindow(), m );
-	    QString n = "PopupMenuEditor";
+	    QString n = "popupMenu";
 	    m->formWindow()->unify( s, n, TRUE );
 	    s->setName( n );
 	    MetaDataBase::addEntry( s );
@@ -333,7 +333,8 @@ void PopupMenuEditor::insert( PopupMenuEditorItem * item, int index )
 	    currentIndex = index;
     }
     item->m = this;
-    item->s->parentMenu = this;
+    if (item->s)
+        item->s->parentMenu = this;
     resizeToContents();
     if ( isVisible() && parentMenu )
 	parentMenu->update(); // draw arrow in parent menu
@@ -420,9 +421,6 @@ void PopupMenuEditor::cut( int index )
 {
     int idx = ( index == -1 ? currentIndex : index );
 
-    if ( clipboardItem && clipboardOperation == Cut )
-	delete clipboardItem;
-
     clipboardOperation = Cut;
     clipboardItem = itemList.at( idx );
 
@@ -442,9 +440,6 @@ void PopupMenuEditor::copy( int index )
 {
     int idx = ( index == -1 ? currentIndex : index );
 
-    if ( clipboardItem && clipboardOperation == Cut )
-	delete clipboardItem;
-
     clipboardOperation = Copy;
     clipboardItem = itemList.at( idx );
 
@@ -855,8 +850,7 @@ void PopupMenuEditor::dropEvent( QDropEvent * e )
 	PopupMenuEditorItemPtrDrag::decode( e, &i );
     } else {
 	if ( e->provides( "application/x-designer-actiongroup" ) ) {
-	    QString s( e->encodedData( "application/x-designer-actiongroup" ) );
-	    QActionGroup * g = (QDesignerActionGroup*)s.toLong();
+	    QActionGroup * g = ::qt_cast<QDesignerActionGroup*>(ActionDrag::action());
 	    if ( g->usesDropDown() ) {
 		i = new PopupMenuEditorItem( g, this );
 		QString n = QString( g->name() ) + "Item";
@@ -876,8 +870,7 @@ void PopupMenuEditor::dropEvent( QDropEvent * e )
 		dropInPlace( g, e->pos().y() );
 	    }
 	} else if ( e->provides( "application/x-designer-actions" ) ) {
-	    QString s( e->encodedData( "application/x-designer-actions" ) );
-	    QAction * a = (QDesignerAction*)s.toLong();
+	    QAction *a = ::qt_cast<QDesignerAction*>(ActionDrag::action());
 	    i = new PopupMenuEditorItem( a, this );
 	}
     }
@@ -1012,12 +1005,12 @@ void PopupMenuEditor::focusInEvent( QFocusEvent * )
 void PopupMenuEditor::focusOutEvent( QFocusEvent * )
 {
     QWidget * fw = qApp->focusWidget();
-    if ( !fw || ( !fw->inherits( "PopupMenuEditor" ) && fw != lineEdit ) ) {
+    if ( !fw || ( !::qt_cast<PopupMenuEditor*>(fw) && fw != lineEdit ) ) {
 	hideSubMenu();
-	if ( fw && fw->inherits( "MenuBarEditor" ) )
+	if ( fw && ::qt_cast<MenuBarEditor*>(fw) )
 	    return;
 	QWidget * w = this;
-	while ( w && w != fw && w->inherits( "PopupMenuEditor" ) ) { // hide all popups
+	while ( w && w != fw && ::qt_cast<PopupMenuEditor*>(w) ) { // hide all popups
 	    w->hide();
 	    w = ((PopupMenuEditor *)w)->parentEditor();
 	}
@@ -1239,12 +1232,15 @@ void PopupMenuEditor::dropInPlace( PopupMenuEditorItem * i, int y )
 
 void PopupMenuEditor::dropInPlace( QActionGroup * g, int y )
 {
+    if (!g->children())
+	return;
     QObjectList l = *g->children();
-    for ( QAction *a = (QAction *)l.last(); a; a = (QAction *)l.prev() ) {
-	QActionGroup *g = ::qt_cast<QActionGroup*>(a);
+    for ( int i = 0; i < (int)l.count(); ++i ) {
+	QAction *a = ::qt_cast<QAction*>(l.at(i));
+	QActionGroup *g = ::qt_cast<QActionGroup*>(l.at(i));
 	if ( g )
 	    dropInPlace( g, y );
-	else
+	else if ( a )
 	    dropInPlace( new PopupMenuEditorItem( a, this ), y );
     }
 }
@@ -1406,13 +1402,17 @@ void PopupMenuEditor::leaveEditMode( QKeyEvent * e )
     if ( currentIndex >= (int)itemList.count() ) {
 	// new item was created
 	QAction * a = formWnd->mainWindow()->actioneditor()->newActionEx();
-	a->setText( lineEdit->text() );
-	a->setMenuText( lineEdit->text() );
+	QString actionText = lineEdit->text();
+	actionText.replace("&&", "&");
+	QString menuText = lineEdit->text();
+	a->setText( actionText );
+	a->setMenuText( menuText );
 	i = createItem( a );
 	QString n = constructName( i );
 	formWindow()->unify( a, n, TRUE );
 	a->setName( n );
 	MetaDataBase::addEntry( a );
+	MetaDataBase::setPropertyChanged( a, "menuText", TRUE );
 	ActionEditor *ae = (ActionEditor*)formWindow()->mainWindow()->child( 0, "ActionEditor" );
 	if ( ae )
 	    ae->updateActionName( a );
