@@ -1,5 +1,5 @@
 /**********************************************************************
-** Copyright (C) 2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 2005 Trolltech AS.  All rights reserved.
 **
 **1 This file is part of Qt Designer.
 **
@@ -337,6 +337,8 @@ QStringList parse_multiline_part( const QString &contents, const QString &key, i
     if ( start )
 	*start = -1;
     QString lastWord;
+    // Account for things like win32: SOURCES
+    int extraWhiteSpaceCount = 0;
     int braceCount = 0;
     for ( int i = 0; i < (int)contents.length(); ++i ) {
 	QChar c( contents[ i ] );
@@ -349,9 +351,21 @@ QStringList parse_multiline_part( const QString &contents, const QString &key, i
 	    braceCount--;
 	    lastWord = "";
 	    break;
-	case ' ': case '\t': case '\\': case '\n':
+	case ' ': case '\t':
+	    if (!key.startsWith(lastWord)) {
+		lastWord = "";
+		extraWhiteSpaceCount = 0;
+	    } else {
+		extraWhiteSpaceCount++;
+	    }
+	    break;
+	case '\\': case '\n':
 	    lastWord = "";
 	    break;
+        case '#':
+            while ( contents[ i ] != '\n' && i < (int)contents.length() )
+                ++i;
+            break;
 	default:
 	    lastWord += c;
 	}
@@ -359,7 +373,7 @@ QStringList parse_multiline_part( const QString &contents, const QString &key, i
 	// ### we should read the 'bla { SOURCES= ... }' stuff as well (braceCount > 0)
 	if ( lastWord == key && braceCount == 0 ) {
 	    if ( start )
-		*start = i - lastWord.length() + 1;
+		*start = i - lastWord.length() - extraWhiteSpaceCount + 1;
 	    QStringList lst;
 	    bool inName = FALSE;
 	    QString currName;
@@ -372,9 +386,26 @@ QStringList parse_multiline_part( const QString &contents, const QString &key, i
 		    hadEqual = TRUE;
 		    continue;
 		}
-		if ( ( c.isLetter() || c.isDigit() || c == '.' || c == '/' || c == '_' || c == '\\' ||
-		       c == '\"' || c == '\'' || c == '=' ||
-		       c == '$' || c == '-' || c == '(' || c == ')' || c == ':'  || c == '+' || c == ',' || c == '~' ) &&
+		if ( ( c.isLetter() ||
+                       c.isDigit() ||
+                       c == '.' ||
+                       c == '/' ||
+                       c == '_' ||
+                       c == '\\' ||
+		       c == '\"' ||
+                       c == '\'' ||
+                       c == '=' ||
+                       c == '$' ||
+                       c == '-' ||
+                       c == '(' ||
+                       c == ')' ||
+                       c == ':' ||
+                       c == '+' ||
+                       c == ',' ||
+                       c == '~' ||
+                       c == '{' ||
+                       c == '}' ||
+                       c == '@' ) &&
 		     c != ' ' && c != '\t' && c != '\n' ) {
 		    if ( !inName )
 			currName = QString::null;
@@ -448,6 +479,8 @@ void Project::parse()
     readPlatformSettings( contents, "LIBS", lbs );
     readPlatformSettings( contents, "INCLUDEPATH", inclPath );
     readPlatformSettings( contents, "DEFINES", defs );
+    readPlatformSettings( contents, "SOURCES", sources );
+    readPlatformSettings( contents, "HEADERS", headers );
 
     LanguageInterface *iface = MetaDataBase::languageInterface( lang );
     if ( iface ) {
@@ -536,7 +569,9 @@ QString Project::makeAbsolute( const QString &f )
 {
     if ( isDummy() )
 	return f;
-    QUrl u( QFileInfo( filename ).dirPath( TRUE ), f );
+    QString encodedUrl = QFileInfo( filename ).dirPath( TRUE );
+    QUrl::encode( encodedUrl );
+    QUrl u( encodedUrl, f );
     return u.path();
 }
 
@@ -544,10 +579,16 @@ QString Project::makeRelative( const QString &f )
 {
     if ( isDummy() )
 	return f;
-    QString p = QFileInfo( filename ).dirPath( TRUE );
+    QString p = QFileInfo( filename ).dirPath( TRUE );    
     QString f2 = f;
+#if defined(Q_WS_WIN32)
+    if ( p.endsWith("/") )
+        p = p.left( p.length() - 1 );
+    if ( f2.left( p.length() ).lower() == p.lower() )
+#else
     if ( f2.left( p.length() ) == p )
-	f2.remove( 0, p.length() + 1 );
+#endif
+	    f2.remove( 0, p.length() + 1 );
     return f2;
 }
 
@@ -565,13 +606,17 @@ static void remove_contents( QString &contents, const QString &s )
 
 static void remove_multiline_contents( QString &contents, const QString &s, int *strt = 0 )
 {
-    int i;
-    parse_multiline_part( contents, s, &i );
+    int i = contents.find( s );
+    int startBrace = contents.findRev("{", i);
+    int endBrace = contents.findRev("}", i);
+    // We found a startBrace before i, but no endBrace after the startBrace before i
+    if (startBrace != -1 && endBrace < startBrace)
+        return;
     if ( strt )
-	*strt = i;
+        *strt = i;
     int start = i;
-    bool lastWasBackspash = FALSE;
-    if ( i != -1 && ( i == 0 || contents[ i - 1 ] != '{' || contents[ i - 1 ] != ':' ) ) {
+    bool lastWasBackspash = TRUE;
+    if ( i != -1 && ( i == 0 || contents[ i - 1 ] != '{' || contents[ i - 1 ] != ':' ) && !(i > 0 && contents[i-1] != '\n')) {
 	for ( ; i < (int)contents.length(); ++i ) {
 	    if ( contents[ i ] == '\n' && !lastWasBackspash )
 		break;
@@ -585,7 +630,12 @@ static void remove_multiline_contents( QString &contents, const QString &s, int
 void Project::save( bool onlyProjectFile )
 {
     bool anythingModified = FALSE;
+
+    //  save sources and forms
     if ( !onlyProjectFile ) {
+
+	saveConnections();
+
 	for ( SourceFile *sf = sourcefiles.first(); sf; sf = sourcefiles.next() ) {
 	    anythingModified = anythingModified || sf->isModified();
 	    if ( !sf->save() )
@@ -612,58 +662,50 @@ void Project::save( bool onlyProjectFile )
     }
 
     QFile f( filename );
-    QString contents;
+    QString original = "";
+
+    // read the existing file
+    bool hasPreviousContents = FALSE;
     if ( f.open( IO_ReadOnly ) ) {
 	QTextStream ts( &f );
-	contents = ts.read();
+	original = ts.read();
 	f.close();
-    } else {
-	// initial contents
-	contents =
-	    "unix {\n"
-	    "  UI_DIR = .ui\n"
-	    "  MOC_DIR = .moc\n"
-	    "  OBJECTS_DIR = .obj\n"
-	    "}\n";
+        hasPreviousContents = TRUE;
+	remove_contents( original, "{SOURCES+=" ); // ### compatibility with early 3.0 betas
+	remove_contents( original, "DBFILE" );
+	remove_contents( original, "LANGUAGE" );
+	remove_contents( original, "TEMPLATE" );
+	removePlatformSettings( original, "CONFIG" );
+	removePlatformSettings( original, "DEFINES" );
+	removePlatformSettings( original, "LIBS" );
+	removePlatformSettings( original, "INCLUDEPATH" );
+	removePlatformSettings( original, "SOURCES" );
+	removePlatformSettings( original, "HEADERS" );
+	remove_multiline_contents( original, "FORMS" );
+	remove_multiline_contents( original, "INTERFACES" ); // compatibility
+	remove_multiline_contents( original, "IMAGES" );
+	for ( QStringList::Iterator it = csList.begin(); it != csList.end(); ++it )
+	    remove_contents( original, *it );
     }
 
-    remove_multiline_contents( contents, "FORMS" );
-    remove_multiline_contents( contents, "INTERFACES" ); // compatibility
-
-    if ( !formfiles.isEmpty() ) {
-	contents += "FORMS\t= ";
-	for ( QPtrListIterator<FormFile> fit = formfiles; fit.current(); ++fit ) {
-	    contents += fit.current()->fileName() +
-		 (fit != formfiles.last() ? " \\\n\t" : "");
-	}
-	contents += "\n";
+    if (!original.isEmpty()) {
+	// Removes any new lines at the beginning of the file
+	while (original.startsWith("\n"))
+	    original.remove(0, 1);
     }
 
-    remove_multiline_contents( contents, "IMAGES" );
-     if ( !pixCollection->isEmpty() ) {
-	contents += "IMAGES\t= ";
-	QValueList<PixmapCollection::Pixmap> pixmaps = pixCollection->pixmaps();
-	for ( QValueList<PixmapCollection::Pixmap>::Iterator it = pixmaps.begin();
-	      it != pixmaps.end(); ++it ) {
-		  contents += makeRelative( (*it).absname );
-		  contents += ++it != pixmaps.end() ? " \\\n\t" : "";
-		  --it;
-	}
-	contents += "\n";
-    }
+    // the contents of the saved file
+    QString contents;
 
-    remove_contents( contents, "{SOURCES+=" ); // ### compatibility with early 3.0 betas
-    remove_contents( contents, "DBFILE" );
-    remove_contents( contents, "LANGUAGE" );
-    remove_contents( contents, "TEMPLATE" );
-    removePlatformSettings( contents, "CONFIG" );
-    removePlatformSettings( contents, "DEFINES" );
-    removePlatformSettings( contents, "LIBS" );
-    removePlatformSettings( contents, "INCLUDEPATH" );
+    // template
+    contents += "TEMPLATE\t= " + templ + "\n";
 
-    contents += "TEMPLATE\t=" + templ + "\n";
+    // language
+    contents += "LANGUAGE\t= " + lang + "\n";
+    contents += "\n";
+
+    // config
     writePlatformSettings( contents, "CONFIG", cfg );
-
     LanguageInterface *iface = MetaDataBase::languageInterface( lang );
     if ( iface ) {
 	QStringList sourceKeys;
@@ -672,15 +714,24 @@ void Project::save( bool onlyProjectFile )
 	    remove_multiline_contents( contents, *spit );
     }
 
+    // libs, defines, includes
+    writePlatformSettings( contents, "LIBS", lbs );
     writePlatformSettings( contents, "DEFINES", defs );
     writePlatformSettings( contents, "INCLUDEPATH", inclPath );
-    writePlatformSettings( contents, "LIBS", lbs );
+    writePlatformSettings( contents, "SOURCES", sources );
+    writePlatformSettings( contents, "HEADERS", headers );
 
-    if ( !dbFile.isEmpty() )
-	contents += "DBFILE\t= " + dbFile + "\n";
-
-    contents += "LANGUAGE\t= " + lang + "\n";
+    // unix
+    if ( !hasPreviousContents ) {
+ 	contents +=
+ 	    "unix {\n"
+ 	    "  UI_DIR = .ui\n"
+ 	    "  MOC_DIR = .moc\n"
+ 	    "  OBJECTS_DIR = .obj\n"
+ 	    "}\n\n";
+    }
 
+    // sources
     if ( !sourcefiles.isEmpty() && iface ) {
 	QMap<QString, QStringList> sourceToKey;
 	for ( SourceFile *f = sourcefiles.first(); f; f = sourcefiles.next() ) {
@@ -699,12 +750,40 @@ void Project::save( bool onlyProjectFile )
 		part += ++sit != lst.end() ? " \\\n\t" : "";
 		--sit;
 	    }
-	    contents.insert( 0, part + "\n" );
+	    part += "\n\n";
+	    contents += part;
 	}
     }
 
+    // forms and interfaces
+    if ( !formfiles.isEmpty() ) {
+	contents += "FORMS\t= ";
+	for ( QPtrListIterator<FormFile> fit = formfiles; fit.current(); ++fit ) {
+	    contents += fit.current()->fileName() +
+		 (fit != formfiles.last() ? " \\\n\t" : "");
+	}
+	contents += "\n\n";
+    }
+
+    // images
+     if ( !pixCollection->isEmpty() ) {
+	contents += "IMAGES\t= ";
+	QValueList<PixmapCollection::Pixmap> pixmaps = pixCollection->pixmaps();
+	for ( QValueList<PixmapCollection::Pixmap>::Iterator it = pixmaps.begin();
+	      it != pixmaps.end(); ++it ) {
+		  contents += makeRelative( (*it).absname );
+		  contents += ++it != pixmaps.end() ? " \\\n\t" : "";
+		  --it;
+	}
+	contents += "\n\n";
+    }
+
+    // database
+    if ( !dbFile.isEmpty() )
+	contents += "DBFILE\t= " + dbFile + "\n";
+
+    // custom settings
     for ( QStringList::Iterator it = csList.begin(); it != csList.end(); ++it ) {
-	remove_contents( contents, *it );
 	QString val = *customSettings.find( *it );
 	if ( !val.isEmpty() )
 	    contents += *it + "\t= " + val + "\n";
@@ -718,11 +797,11 @@ void Project::save( bool onlyProjectFile )
 
     QTextStream os( &f );
     os << contents;
+    if (hasPreviousContents)
+        os << original;
 
     f.close();
 
-    saveConnections();
-
     setModified( FALSE );
 
     if ( singleProjectMode() ) {
@@ -819,20 +898,14 @@ static void saveSingleProperty( QTextStream &ts, const QString& name, const QStr
 }
 #endif
 
-#ifndef QT_NO_SQL
-static bool inSaveConnections = FALSE;
-#endif
 void Project::saveConnections()
 {
 #ifndef QT_NO_SQL
-    if ( inSaveConnections )
-	return;
-    inSaveConnections = TRUE;
-
     if ( dbFile.isEmpty() ) {
 	QFileInfo fi( fileName() );
 	setDatabaseDescription( fi.baseName() + ".db" );
     }
+
     QFile f( makeAbsolute( dbFile ) );
 
     if ( dbConnections.isEmpty() ) {
@@ -840,11 +913,8 @@ void Project::saveConnections()
 	    f.remove();
 	setDatabaseDescription( "" );
 	modified = TRUE;
-	save( TRUE );
-	inSaveConnections = FALSE;
 	return;
     }
-    save( TRUE );
 
     /* .db xml */
     if ( f.open( IO_WriteOnly | IO_Translate ) ) {
@@ -894,8 +964,6 @@ void Project::saveConnections()
 	ts << "</DB>" << endl;
 	f.close();
     }
-
-    inSaveConnections = FALSE;
 #endif
 }
 
@@ -1217,7 +1285,13 @@ void Project::writePlatformSettings( QString &contents, const QString &setting,
 				     const QMap<QString, QString> &input )
 {
     const QString platforms[] = { "", "win32", "unix", "mac", QString::null };
-    for ( int i = 0; platforms[ i ] != QString::null; ++i ) {
+    int i;
+    LanguageInterface *iface = MetaDataBase::languageInterface( lang );
+    if (iface && (setting == "SOURCES" || setting == "HEADERS")) // The (all) part will be saved later on
+        i = 1;
+    else
+	i = 0;
+    for (; platforms[ i ] != QString::null; ++i ) {
 	QString p = platforms[ i ];
 	if ( !p.isEmpty() )
 	    p += ":";
@@ -1227,7 +1301,7 @@ void Project::writePlatformSettings( QString &contents, const QString &setting,
 	QMap<QString, QString>::ConstIterator it = input.find( key );
 	if ( it == input.end() || (*it).isEmpty() )
 	    continue;
-	contents += p + setting + "\t+= " + *it + "\n";
+	contents += p + setting + "\t+= " + *it + "\n\n";
     }
 }
 
