@@ -1,5 +1,5 @@
  /**********************************************************************
-** Copyright (C) 2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of Qt Designer.
 **
@@ -8,10 +8,16 @@
 ** Foundation and appearing in the file LICENSE.GPL included in the
 ** packaging of this file.
 **
+** Licensees holding valid Qt Enterprise Edition or Qt Professional Edition
+** licenses may use this file in accordance with the Qt Commercial License
+** Agreement provided with the Software.
+**
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ** See http://www.trolltech.com/gpl/ for GPL licensing information.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+**   information about Qt Commercial License Agreements.
 **
 ** Contact info@trolltech.com if any conditions of this licensing are
 ** not clear to you.
@@ -22,7 +28,6 @@
 #define VIEWMANAGER_H
 
 #include <qwidget.h>
-#include "dlldefs.h"
 #include <qvaluelist.h>
 
 class QChildEvent;
@@ -31,8 +36,9 @@ class QVBoxLayout;
 class QDockArea;
 class QTextParagraph;
 class QLabel;
+class QTimer;
 
-class EDITOR_EXPORT ViewManager : public QWidget
+class ViewManager : public QWidget
 {
     Q_OBJECT
 
@@ -48,10 +54,11 @@ public:
     void setStackFrame( int line );
     void clearStep();
     void clearStackFrame();
-    void setBreakPoints( const QValueList<int> &l );
-    QValueList<int> breakPoints() const;
+    void setBreakPoints( const QValueList<uint> &l );
+    QValueList<uint> breakPoints() const;
 
     void emitMarkersChanged();
+    MarkerWidget *marker_widget() const { return markerWidget; }
 
 signals:
     void markersChanged();
@@ -60,10 +67,14 @@ signals:
     void collapse( bool all /*else only functions*/ );
     void expand( bool all /*else only functions*/ );
     void editBreakPoints();
+    void isBreakpointPossible( bool &possible, const QString &code, int line );
 
 protected slots:
     void clearErrorMarker();
     void cursorPositionChanged( int row, int col );
+    void showMessage( const QString &msg );
+    void clearStatusBar();
+
 protected:
     void childEvent( QChildEvent *e );
     void resizeEvent( QResizeEvent *e );
@@ -74,6 +85,8 @@ private:
     QVBoxLayout *layout;
     QDockArea *dockArea;
     QLabel *posLabel;
+    QString extraText;
+    QTimer *messageTimer;
 
 };
 
